<?php

require '../../lib/database.inc';
require '../../lib/functions.inc';
require '../includes/config.inc';
require '../models/pricebox_m.inc';
require '../includes/pricebox.inc';
require '../../lib/class-IXR.php';
require '../models/site_m.inc';
require '../models/category_m.inc';
require '../models/export_mappings_m.inc';

set_time_limit(0);
if (0) {
		$db->query('use wp271');
		$db->query('delete from wp_posts where id >= 123');
		$db->query('use datafeed_studio');
		$db->query('delete from datafeed_studio.export_mappings');
}

function shutdown() {
	dslog("Shutdown : End of Script\n");
}

function dslog($message) {
	echo(date(DATE_RFC822) . " : $message");
}

function make_description($product) {

	global $EXPORT_TEMPLATE;

	$description = $EXPORT_TEMPLATE;
	

	$mappings = array(
		'MYKEY' => $product['mykey'],
		'NAME' => $product['name'], 
		'DEEP_LINK' => $product['deep_link'],
		'IMAGE_URL' => $product['image_url'],
		'DESCRIPTION' => $product['description'],
		'SEARCHNAME' => convertNameIntoSearchClause($product['name']),
		'DISPLAY_PRICE' => $product['display_price'],
		'MERCHANT_NAME' => $product['merchant_name'],

	);
	
	foreach($mappings as $key => $value) {
		$description = str_replace("%$key%", $value, $description);
	}
	
	return $description;
}

function convertNameIntoSearchClause($name) {
	global $MYSQL_MIN_WORD_LENGTH;
	
	$name = html_entity_decode($name);
	$name = ereg_replace("[^ A-Za-z0-9]", "", $name); 
	$name = str_squeeze($name);
	
	$words = split(' ', $name);
	$clause = '';
	foreach ($words as $word) {
			if (strlen($word) >= $MYSQL_MIN_WORD_LENGTH) {
				$clause .= "+$word ";					
			}
	}
	
	return $clause;
}

function add_custom_field($custom_fields, $key, $value) {
	if ($key) {

		for ($i = 0; $i < count($custom_fields); $i++) {
			$custom_field = $custom_fields[$i];
			
			echo("Processing id={$custom_field['id']} key={$custom_field['key']} value={$custom_field['value']}\n");
			
			if ($key == $custom_field['key']) {
				echo("Found existing match for $key with id of {$custom_field['id']}\n");
				$custom_fields[$i]['value'] = $value;
				return $custom_fields;
			}
		}		
		
		$cf = array('key' => "$key", 'value' => "$value");
		array_push($custom_fields, $cf);		
	}
	
	return $custom_fields;
	
}

function get_custom_fields($product, $custom_fields = array()) {
	
	global $EXPORT_CF_MYKEY;
	global $EXPORT_CF_NAME;
	global $EXPORT_CF_DEEP_LINK;
	global $EXPORT_CF_IMAGE_URL;
	global $EXPORT_CF_DESCRIPTION;
	global $EXPORT_CF_SEARCHNAME;
	global $EXPORT_CF_DISPLAY_PRICE;
	global $EXPORT_CF_MERCHANT_NAME;
	
	$custom_fields = add_custom_field($custom_fields, $EXPORT_CF_MYKEY, $product['mykey']);
	$custom_fields = add_custom_field($custom_fields, $EXPORT_CF_NAME, $product['name']);
	$custom_fields = add_custom_field($custom_fields, $EXPORT_CF_DEEP_LINK, $product['deep_link']);
	$custom_fields = add_custom_field($custom_fields, $EXPORT_CF_IMAGE_URL, $product['image_url']);
	$custom_fields = add_custom_field($custom_fields, $EXPORT_CF_DESCRIPTION, $product['description']);
	$custom_fields = add_custom_field($custom_fields, $EXPORT_CF_SEARCHNAME, convertNameIntoSearchClause($product['name']));
	$custom_fields = add_custom_field($custom_fields, $EXPORT_CF_DISPLAY_PRICE, $product['display_price']);
	$custom_fields = add_custom_field($custom_fields, $EXPORT_CF_MERCHANT_NAME, $product['merchant_name']);
	
	#print_r($custom_fields);
	return $custom_fields;
}

function create_post($site, $product, $export_cat, $client) {
	dslog("Creating post for {$product['mykey']} ({$product['name']})\n");
	
	$custom_fields = get_custom_fields($product);
	
	$client->query('metaWeblog.newPost', '0', $site['export_username'], $site['export_password'], 
	
	array(
	'title' => $product['name'], 
	'description' => make_description($product), 
	'custom_fields' => $custom_fields,
	'categories' => array($export_cat)), TRUE);
	
	$response = $client->getResponse();
	
	$export_mapping = array();
	$export_mapping['mykey'] = $product['mykey'];
	$export_mapping['export_id'] = $response;
	$export_mapping['site_id'] = $site['id'];
	
	export_mapping_insert($export_mapping);
	//print_r($response);
}

function update_post($site, $product, $export_cat, $client, $current_mapping) {
	dslog("Updating post {$current_mapping['export_id']} ({$product['name']})\n");
	
	$client->query('metaWeblog.getPost', $current_mapping['export_id'], $site['export_username'], $site['export_password']);
	$current_post = $client->getResponse();
//	print_r($current_post);
	
	$current_fields = $current_post['custom_fields'];
	//print_r($current_fields);
	
	$custom_fields = get_custom_fields($product, $current_fields);
		
	$client->query('metaWeblog.editPost', $current_mapping['export_id'], $site['export_username'], $site['export_password'], 
	
	array(
		'title' => $product['name'], 
		'description' => make_description($product), 
		'custom_fields' => $custom_fields,		
		'categories' => array($export_cat)
		), TRUE);
	$response = $client->getResponse();	
	//echo("\nRESP: $response\n");
}

// ****************************************************************************
// Main
// ****************************************************************************

dslog(">>> DS Export process started\n");
register_shutdown_function('shutdown');
$EXPORT_LOCK_FILE = $DATAFILE_FOLDER . '/export.lck';

if (file_exists($EXPORT_LOCK_FILE)) {
	print "An export is already in progress so I wont run again. If you think this is\n";
	print " a mistake the delete the file : \n\n$EXPORT_LOCK_FILE\n\n and run this script again.\n";
	die;
}

chdir("$INSTALL_FOLDER/admin/bin");

dslog("In export.php\n");

$site_id = $argv[1];
if (! $site_id) {
	dslog("You must supply the ID of the site\n");
	exit;
}


$site = site_find($site_id);

include_once("$SITECONFIG_FOLDER/$site_id.inc");

$client = new IXR_Client($site['url']);

$mappings = unserialize($EXPORT_MAPPINGS);

touch($EXPORT_LOCK_FILE);

if ($mappings) {
	foreach ($mappings as $ds_cat_id => $export_cat) {

		$ds_cat = category_find($ds_cat_id);
		dslog("$ds_cat_id ({$ds_cat['name']}) => $export_cat\n");

		$products_result = category_find_products($ds_cat);
		$numfields = $db->field_count;
		$tfields = Array();
		for ($i = 0; $i < $numfields; $i++) {
		  $field =  $products_result->fetch_field_direct($i);
		  $tfields[$i] = $field->table. '.' .$field->name;
	  }

		while ($data = $products_result->fetch_row()) {

		  $row = Array();
		  for ($i = 0; $i < $numfields; $i++) {
		    $row[$tfields[$i]] = $data[$i];
			}

		  $product = array();		
		  $product['mykey'] = $row['p.mykey'];
		  $product['name'] = $row['m.name'] ? $row['m.name'] : $row['p.name'];
		  $product['deep_link'] = $row['m.deep_link'] ? $row['m.deep_link'] : $row['p.deep_link'];
		  $product['image_url'] = $row['m.image_url'] ? $row['m.image_url'] : $row['p.image_url'];
		  $product['description'] = $row['m.description'] ? $row['m.description'] : $row['p.description'];
		  $product['display_price'] = $row['m.display_price'] ? $row['m.display_price'] : $row['p.display_price'];
		  $product['merchant_name'] = $row['m.merchant_name'] ? $row['m.merchant_name'] : $row['p.merchant_name'];

			$current_mapping = export_mapping_findby_mykey_and_site_id($product['mykey'], $site['id']);

			if ($current_mapping) {
				if ($site['export_update_existing']) {
					update_post($site, $product, $export_cat, $client, $current_mapping);					
				} else {
					dslog("Ignoring post {$current_mapping['export_id']} ({$product['name']}) - already exported\n");
				}	
			} else {
				create_post($site, $product, $export_cat, $client);
			}
		}	
	}	
} else {
	dslog("ERROR : No Mappings Defined?\n");
}

unlink($EXPORT_LOCK_FILE);
dslog("<<< DS Export process finished\n");

?>
