<?php
require '../includes/config.inc';
require '../models/datafeed_m.inc';
require '../models/parser_m.inc';
require '../models/filter_m.inc';
require '../models/product_m.inc';
require '../models/category_match_m.inc';
require '../models/category_m.inc';
require '../includes/common.inc';

set_time_limit(0);

$CATEGORY_MATCHES = array();
$SUB_CATEGORY_MATCHES = array();

function shutdown() {
	dslog("Shutdown : End of Script\n");
}

function dslog($message) {
	echo(date(DATE_RFC822) . " : $message");
}

function safe_die($message) {
	global $LOCK_FILE;
	
	if (file_exists($LOCK_FILE)) {
		unlink($LOCK_FILE);
	}
	
	die($message);
}

function resultset_to_array($resultset) {
	$items = array();
	
	while ($item = $resultset->fetch_array()) {
		array_push($items, $item);
	}
	
	return $items;
}

function createTemporaryProductsTable() {
	global $db;
	dslog("Creating temporary products table\n");
	dslog("...dropping temporary products table\n");
	$db->query("drop table if exists temp_products");			
	dslog("...creating temporary products table\n");
	$db->query("create table temp_products like products");
	dslog("...disabling keys on temporary products\n");	
	$db->query("alter table temp_products disable keys");

}

function copyProductsTableToTemporary($excluded_datafeed_ids) {
	global $db;
	dslog ("Copying products (except from IDs $excluded_datafeed_ids) into temporary table\n");
	$sql = "insert into temp_products select * from products where datafeed_id not in ($excluded_datafeed_ids) ";
	
	dslog("$sql\n");
	
	$result = $db->query($sql);

	if (! $result) {
		dslog("Error" . $db->error);
	}
}

function copyTemporaryToProductsTable() {
	global $db;
	dslog("Copying temporary products to main products table\n");
	dslog("...enabling keys on products table\n");
	$db->query("alter table temp_products enable keys");	
	dslog("...dropping real products table\n");	
	$db->query("drop table products");
	dslog("...renaming temporary products table to real products table\n");	
	$db->query("rename table temp_products to products");
	dslog("...dropping temporary products table\n");	
	$db->query("drop table temp_products");
}
	
function copyNonFeedProductsToTemporary() {
	global $db;
	$db->query ("insert into temp_products select * from products where datafeed_id < 0 or ws_api_key is not null or is_custom = 1");	
}

function fetchFeed($datafeed) {
	global $DATAFILE_FOLDER;
	global $PHP_SHELL_COMMAND;
	global $SCRIPT_FOLDER;
	global $CATEGORY_MATCHES;	
	global $SUB_CATEGORY_MATCHES;	
		
	dslog("---------------------------------------------------------------\n");
	dslog(date('l jS \of F Y h:i:s A'));		
	dslog("Processing datafeed ID {$datafeed['id']} ({$datafeed['merchant_name']})\n");

	if ($datafeed['feed_type'] == 'automatic') {
		$datafeed['status'] = "Downloading";
		datafeed_update($datafeed);
	
		dslog ("Feed Type is Automatic. Downloading datafeed file\n");
		$datafile = $datafeed['id'] . ".dsf";

    $ch = curl_init($datafeed['url']);
    $fp = fopen("$DATAFILE_FOLDER/$datafile", "w");

    curl_setopt($ch, CURLOPT_FILE, $fp);
    curl_setopt($ch, CURLOPT_HEADER, 0);
		curl_setopt ($ch, CURLOPT_CONNECTTIMEOUT, 60);
		curl_setopt ($ch, CURLOPT_TIMEOUT, 300);

    curl_exec($ch);

		if(curl_errno($ch)) {
			dslog('WARNING! Curl error: #' . curl_errno($ch) . ' - '. curl_error($ch) . "\n");
		}
		
    curl_close($ch);
    fclose($fp);   

		chmod("$DATAFILE_FOLDER/$datafile", 0777);
		
		$parser = parser_find($datafeed['parser_id']);
		if ($parser['post_download_script']) {
			dslog ("Executing post download script {$parser['post_download_script']}\n");
			
			if (substr($parser['post_download_script'], 0, 1) != '/') {
				$scriptFolder = $SCRIPT_FOLDER . '/';
			} else {
				$scriptFolder = '';
			}
			
			system("$scriptFolder{$parser['post_download_script']} $DATAFILE_FOLDER/$datafile");
		} else {
			dslog("No post download script specified.\n");
		}
		
	} else {
		$datafile = $datafeed['datafile'];
		dslog ("Feed Type is Manual. Using file already uploaded ($datafile)\n");
	}
	
	return $datafile;
}

function populateMatches($category_type) {
	$matches = array();	
	
	$category_matches_array = resultset_to_array(category_match_find_all($category_type));	
	foreach($category_matches_array as $match) {
		$matches[$match['match_content']] = $match;
	}
	
	return $matches;
}

function parseFeed($datafeed, $datafile, $import_start_time) {
	global $DATAFILE_FOLDER;
	global $CATEGORY_MATCHES;
	global $SUB_CATEGORY_MATCHES;
	global $DS_IMPORT_DELETE_AFTER;
	
	$datafeed['status'] = "Importing";
	$datafeed['total_products'] = 0;
	datafeed_update($datafeed);
	
	$parser = parser_find($datafeed['parser_id']);
	$filter_result = filter_findby_parser_id($datafeed['parser_id']);	
	$filters = array();
	
	for ($i = 0; $i < $filter_result->num_rows; $i++) {

		$filter = $filter_result->fetch_array();	
		$field_array = $filters[$filter['field_name']];
		
		dslog ("Adding filter for {$filter['field_name']}\n");
		
		if (! $field_array) {
			dslog("No field array for {$filter['field_name']} - creating\n");
			$field_array = array();
			$field_array[$filter['id']] = $filter;
		}
		
		$field_array[$filter['id']] = $filter;
		$filters[$filter['field_name']] = $field_array;					
	}
		
	#var_dump($filters);
		
	$fp = fopen($DATAFILE_FOLDER . '/' . $datafile, "r");
	
	if (! $fp) {
		dslog("***********************************************************\n");
		dslog("WARNING - Could not open the feed file for reading!\n");
		dslog ("Import finished for {$datafeed['merchant_name']}. 0 products imported\n");
		$datafeed['status'] = "FAILED - NO FILE FOUND";
		$datafeed['total_products'] = 0;	
		$datafeed['last_import'] = date('Y-m-d H:i:s', time());
		datafeed_update($datafeed);
		dslog("***********************************************************\n");
		return;
	}
	
	if ($parser['skip_first_row'] == "on") {
		dslog("Skipping First Line\n");
		fgets($fp);
	}

	$count = 0;
	$failed = 0;
	
	$sep = str_replace('\t', "\t", $parser['parser_separator']);
		
	$CATEGORY_MATCHES = populateMatches(CATEGORY);
	$SUB_CATEGORY_MATCHES = populateMatches(SUB_CATEGORY);
		
	while ($csv_line = fgetcsv($fp, 10000, $sep)) {
		
		if (product_insert(create_product($csv_line, $datafeed, $parser, $filters), true)) {
			$count++;
		} else {
			$failed++;
		}

		if (($count % 100) == 0) {
			dslog("Imported $count products for {$datafeed['merchant_name']}\n");
			$datafeed['total_products'] = $count;
			datafeed_update($datafeed);
 		}
	}
	
	dslog ("Import finished for {$datafeed['merchant_name']}. $count products imported (and $failed failed to be imported)\n");
	$datafeed['status'] = "Imported";
	$datafeed['total_products'] = $count;	
	$datafeed['last_import'] = date('Y-m-d H:i:s', time());
	datafeed_update($datafeed);
	
	if (($DS_IMPORT_DELETE_AFTER) && ($datafeed['feed_type'] == 'automatic')) {
		dslog("Deleting feed file\n");
		unlink($DATAFILE_FOLDER . '/' . $datafile);
	}
	
}

function filter_replace($value, $param1, $param2) {
	return str_replace($param1, $param2, $value);
}

function filter_buyatstock($value) {
	return (($value == 'In Stock') || ($value == '1') || ($value == '') || ($value == 'Y'));
}

function set_value($product, $parser, $csv_line, $filters, $field) {

	$field_filters = $filters[$field];
	#dslog "Found " . sizeof($field_filters) . " matching filters for '$field'\n";
	
	if ($field_filters) {
		$value = $csv_line[$parser[$field]];
		
		#dslog "Old value '$value'\n";
		
		foreach ($filters[$field] as $filter) {
			#dslog "{$filter['id']} - {$filter['filter_type']}\n";
			
			if ($filter['filter_type'] == 'replace') {
				$value = filter_replace($value, $filter['param1'], $filter['param2']);
			} else if ($filter['filter_type'] == 'buyatstock') {
				$value = filter_buyatstock($value);
			} else if ($filter['filter_type'] == 'porstock') {
				$value = 1;
			} else if ($filter['filter_type'] == 'append') {
				$value = $value . $filter['param1'];
			} else if ($filter['filter_type'] == 'prepend') {
				$value = $filter['param1'] . $value;
			}
		}

		#dslog "New value '$value'\n";
	} else {
		$value = $csv_line[$parser[$field]];
	}
	
	#dslog "$field = '$value'\n";
	$product[$field] = $value;
	return $product;
}

function set_product_category($product, $category_type, &$category_matches) {
	
	if ($category_type == CATEGORY) {
		$category_name = $product['category'];
		$category_key = 'category_id';
	} elseif ($category_type == SUB_CATEGORY) {
		$category_name = $product['subcategory'];
		$category_key = 'sub_category_id';
	} else {
		safe_die("Unknown Category Type : $category_type");
	}

	if (empty($category_name)) {
		return -1;
	}

	//dslog("Category = $category_name\n");
	
	$existing_match = $category_matches[strtolower($category_name)];
	
	if ($existing_match) {		
		//dslog("Found a match for $category_name \n");
		return $existing_match['category_id'];
	} else {
		//dslog("Creating new category for $category_name\n");
		
		// create new category
		$category = array();
		$category['name'] = $category_name;
		$category['cat_type'] = 'auto';
		
		if ($category_type == SUB_CATEGORY) {
			//dslog("Sub Cat - setting parent id to " . $product['category_id'] . "\n");
			$category['parent_category_id'] = $product['category_id'];
		}
		
		$category_id = category_insert($category);
		//dslog("New cat ID = " . $category_id . "\n");
		
		// create new match
		$match = array();
		$match['category_id'] = $category_id;
		$match['match_content'] = strtolower($category_name);
		$match['category_type'] = $category_type;
		category_match_insert($match);
		$category_matches[strtolower($category_name)] = $match;
		return $category_id;
	}	
}


function create_product($csv_line, $datafeed, $parser, $filters) {		
	
	global $CATEGORY_MATCHES;
	global $SUB_CATEGORY_MATCHES;
	
	$product = array();

	$product = set_value($product, $parser, $csv_line, $filters, 'category');
	$product = set_value($product, $parser, $csv_line, $filters, 'subcategory');
	$product = set_value($product, $parser, $csv_line, $filters, 'model_number');
	$product = set_value($product, $parser, $csv_line, $filters, 'name');
	$product = set_value($product, $parser, $csv_line, $filters, 'description');		
	$product = set_value($product, $parser, $csv_line, $filters, 'thumb_url');		
	$product = set_value($product, $parser, $csv_line, $filters, 'image_url');	
	$product = set_value($product, $parser, $csv_line, $filters, 'deep_link');	
	$product = set_value($product, $parser, $csv_line, $filters, 'delivery_time');	
	$product = set_value($product, $parser, $csv_line, $filters, 'in_stock');											
	$product = set_value($product, $parser, $csv_line, $filters, 'rrp_price');		
	$product = set_value($product, $parser, $csv_line, $filters, 'display_price');		
	$product = set_value($product, $parser, $csv_line, $filters, 'is_preorder');		
	$product = set_value($product, $parser, $csv_line, $filters, 'delivery_cost');		
	$product = set_value($product, $parser, $csv_line, $filters, 'mykey');		
	$product = set_value($product, $parser, $csv_line, $filters, 'userfield1');		
	$product = set_value($product, $parser, $csv_line, $filters, 'userfield2');		
	$product = set_value($product, $parser, $csv_line, $filters, 'userfield3');		
	$product = set_value($product, $parser, $csv_line, $filters, 'userfield4');		
		
	$product['merchant_name'] = $datafeed['merchant_name'];
	$product['mykey'] = $datafeed['id'] . "_" . slug($product['mykey']);
	$product['datafeed_id'] = $datafeed['id'];
	
	
	if ($parser['in_stock'] == -1) {
		$product['in_stock'] = -1;
	}
	
	$mysqldate = date('Y-m-d H:i:s', time());		
	$product['updated_at'] = $mysqldate;
	$product['created_at'] = $mysqldate;
	
	$product['autoslug'] = slug($product['name']);
	
	
	$product['category_id'] = set_product_category($product, CATEGORY, $CATEGORY_MATCHES);
	$product['sub_category_id'] = 
		set_product_category($product, SUB_CATEGORY, $SUB_CATEGORY_MATCHES);
		
	return $product;
}

function processFeeds($datafeed_result) {
	for ($i = 0; $i < $datafeed_result->num_rows; $i++) {
		processFeed($datafeed_result->fetch_array());		
	}	
}

function processFeed($datafeed) {
	if ($datafeed['feed_type'] != 'webservice') {
		
		if ($datafeed['is_suspended']) {
			dslog("******************************************************\n");
			dslog("Skipping SUSPENDED feed (ID : {$datafeed['id']}, Name : {$datafeed['merchant_name']})\n");
			dslog("******************************************************\n");
			$datafeed['status'] = 'SUSPENDED';
			$datafeed['total_products'] = 0;
			datafeed_update($datafeed);
		}	else {
			$datafile = fetchFeed(datafeed_find($datafeed['id']));
			parseFeed($datafeed, $datafile, $import_start_time);				
		}
	}
}

// ****************************************************************************
// Main
// ****************************************************************************

dslog(">>> DS import.php process started\n");
register_shutdown_function('shutdown');

if (file_exists($LOCK_FILE)) {
	print "An import is already in progress so I wont run again. If you think this is\n";
	print " a mistake the delete the file : \n\n$LOCK_FILE\n\n and run this script again.\n";
	die;
}

chdir("$INSTALL_FOLDER/admin/bin");

if ($argc != 2) {
	print "Usage: import.php [datafeed_id|all|datafeed_id_1,datafeed_id_2,...|a-z]\n";
	die;
}

$import_start_time = time();
touch($LOCK_FILE);
createTemporaryProductsTable();

if ($argv[1] == "all") {
	dslog("ALL Feed Mode\n");	
	copyNonFeedProductsToTemporary();
	processFeeds(datafeed_find_all());
} elseif (strpos($argv[1], "-")) {
	dslog("AZ Feed Mode\n");
	list($start_letter, $end_letter) = explode('-', $argv[1]);	
	dslog("Processing feeds $start_letter -> $end_letter\n");
	copyProductsTableToTemporary(datafeed_find_ids_az($start_letter, $end_letter));
	processFeeds(datafeed_find_az($start_letter, $end_letter));	
} elseif (strpos($argv[1], ",")) {
	dslog("Multiple Feed Mode\n");	
	copyProductsTableToTemporary($argv[1]);
	processFeeds(datafeed_find_by_ids($argv[1]));	
} else {
	dslog("Single Feed Mode\n");	
	copyProductsTableToTemporary($argv[1]);
	processFeed(datafeed_find($argv[1]));
}

copyTemporaryToProductsTable();

if ($DS_NO_AUTO_CATEGORIES) {
	dslog("Deleting auto categories\n");
	$db->query("delete from category where cat_type = 'auto'");	
	$db->query("delete from category_matches");	
}

unlink($LOCK_FILE);
dslog("<<< DS import.php process finished\n");

?>
