<?php 

require_once 'product_api.class.php';

class AffiliateWindow extends ProductAPI {

	var $config = null;
	
	function AffiliateWindow($config) {
		$this->config = $config;
	}
	
	function getKey() {
		return "AWIN";
	}
	
	function generateAPIKey($product) {
		return $this->getKey() . '_' . $product->iMerchantId;	
	}
	
	function generateProductKey($product) {
		return $this->getKey() . '_' . $product->iId;			
	}
	
	function curl_file_get_contents($request) {

		$ch = curl_init();
		$timeout = 60; // set to zero for no timeout
		curl_setopt ($ch, CURLOPT_URL, "http://v3.core.com.productserve.com/ProductServeService");
		curl_setopt ($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt ($ch, CURLOPT_CONNECTTIMEOUT, $timeout);
		curl_setopt ($ch, CURLOPT_HTTPHEADER, array('SOAPAction: ""', 'Content-Type: text/xml; charset=utf-8')); 	
		curl_setopt ($ch, CURLOPT_POSTFIELDS, $request); 	
		curl_setopt ($ch, CURLOPT_POST, 1	); 	

		$response = curl_exec($ch);
		//echo("<textarea>$response</textarea>");
		curl_close($ch);
		return $response;
	}

	function makePopulatedDSProduct($product) {
		$p = array();
		$p['mykey'] 							= $this->generateProductKey($product);
		$p['network_product_key']	= $product->iId;
		$p['name'] 								= $product->sName;
		$p['description'] 				= $product->sDescription;
		$p['display_price'] 			= $product->fPrice;
		$p['deep_link']						= $product->sAwDeepLink;  
		$p['thumb_url']						= $product->sMerchantImageUrl;  
		$p['image_url']						= $product->sMerchantImageUrl;  
		$p['ws_api_key']					= $this->getKey();
		$p['currency_prefix']			= '&pound;';


		//$p['category_id']					= $product->iCategoryId;
		//$p['merchant_id']					= $product->iMerchantId;

		$datafeed = datafeed_findby_apikey($this->generateAPIKey($product));
		
		if ($datafeed) {
			$p['datafeed_id'] = $datafeed['id'];
		} else {
			$merchant = $this->getMerchant($product->iMerchantId);

			$datafeed = array();
			
			$datafeed['merchant_name']  = $merchant->sName;
			$datafeed['merchant_logo']  = $merchant->sLogoUrl;
			$datafeed['parser_id'] 		  = -1;
			$datafeed['description'] 	  = $merchant->sDescription; 
			$datafeed['datafile']			  = '';
			$datafeed['url']					  = '';
			$datafeed['feed_type']	 	  = 'webservice';
			$datafeed['status']					= 'N/A';
			$datafeed['total_products'] = -1;
			$datafeed['api_key']				  = $this->generateAPIKey($product);
			$datafeed['parser_id']			= -1;
			$datafeed['country_code']  = 'GBR';
			$datafeed['is_suspended']		= 0;
			
			
			datafeed_insert($datafeed);
		}

		$p['merchant_name'] = $datafeed['merchant_name'];
		return $p;
	}
	
	public function findProducts($options) {
		
		if ($options['category_id'] != '-1') {
			$category_filter = <<<EOFILTER
			<bIncludeTree>true</bIncludeTree>
			<oActiveRefineByGroup>
				<iId>4</iId>
				<sName/>
				<oRefineByDefinition>
					<sId>{$options['category_id']}</sId>
					<sName/>
				</oRefineByDefinition>			
			</oActiveRefineByGroup>
EOFILTER;
		}
		
		$request = <<<EOREQUEST
<?xml version="1.0" encoding="UTF-8"?>
  <SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="http://v3.core.com.productserve.com/" xmlns:ns2="http://api.productserve.com/">
    <SOAP-ENV:Header>
      <ns2:UserAuthentication SOAP-ENV:mustUnderstand="1" SOAP-ENV:actor="http://api.productserve.com/">
			<sApiKey>{$this->config['password']}</sApiKey>
			</ns2:UserAuthentication>
		</SOAP-ENV:Header>
		<SOAP-ENV:Body>
			<ns1:getProductList>
				<sQuery>{$options['keywords']}</sQuery>
				<sColumnToReturn>sMerchantImageUrl</sColumnToReturn>				
				<iLimit>{$this->config['max_results']}</iLimit>
				$category_filter
		</ns1:getProductList>
  </SOAP-ENV:Body>
</SOAP-ENV:Envelope>
EOREQUEST;
	
		$products = array();
		$response = $this->curl_file_get_contents($request);
		$xml = simplexml_load_string($response);

		foreach($xml->xpath('//ns1:getProductListResponse/oProduct') as $product) {
			$products[] = $this->makePopulatedDSProduct($product);
		}

		return $products;
	}
	
	public function getCategories($options) {
		$request = <<<EOREQUEST
<?xml version="1.0" encoding="UTF-8"?>
	<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="http://v3.core.com.productserve.com/" xmlns:ns2="http://api.productserve.com/">
	<SOAP-ENV:Header>
		<ns2:UserAuthentication SOAP-ENV:mustUnderstand="1" SOAP-ENV:actor="http://api.productserve.com/">
			<sApiKey>{$this->config['password']}</sApiKey>
		</ns2:UserAuthentication>
	</SOAP-ENV:Header>
		
		<SOAP-ENV:Body>
			<ns1:getCategoryTree/>
		</SOAP-ENV:Body>
</SOAP-ENV:Envelope>
EOREQUEST;

		$categories = array();

		$cat = array();
		$cat['id'] = '-1';
		$cat['name'] = 'ALL';
		$categories[] = $cat;

		$response = $this->curl_file_get_contents($request);
		$xml = simplexml_load_string($response);
		
		foreach($xml->xpath('//ns1:getCategoryTreeResponse/oCategoryTree/oCategory') as $category) {
			$cat = array();
			$cat['id'] = $category->iId;
			$cat['name'] = $category->sName;
			$categories[] = $cat;
		}

		return $categories;
	}

	public function getMerchants($options) {
		
	}
	
	public function getProduct($options) {

		$request = <<<EOREQUEST
<?xml version="1.0" encoding="UTF-8"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="http://v3.core.com.productserve.com/" xmlns:ns2="http://api.productserve.com/">

	<SOAP-ENV:Header>
		<ns2:UserAuthentication SOAP-ENV:mustUnderstand="1" SOAP-ENV:actor="http://api.productserve.com/">
			<sApiKey>{$this->config['password']}</sApiKey>
		</ns2:UserAuthentication>
	</SOAP-ENV:Header>
	<SOAP-ENV:Body>
		<ns1:getProduct>
			<iProductId>{$options['product_key']}</iProductId>
			<sColumnToReturn>sDescription</sColumnToReturn>
			<sColumnToReturn>sMerchantImageUrl</sColumnToReturn>
		</ns1:getProduct>
	</SOAP-ENV:Body>
</SOAP-ENV:Envelope>
EOREQUEST;

		$response = $this->curl_file_get_contents($request);
		$xml = simplexml_load_string($response);
		
		$products = $xml->xpath('//ns1:getProductResponse/oProduct');
		return $this->makePopulatedDSProduct($products[0]);
}

	
	public function getMerchant($merchant_id) {
		$request = <<<EOREQUEST
<?xml version="1.0" encoding="UTF-8"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="http://api.productserve.com/">
	<SOAP-ENV:Header>
		<ns1:UserAuthentication SOAP-ENV:mustUnderstand="1" SOAP-ENV:actor="http://api.productserve.com/">
			<iId>{$this->config['userid']}</iId>
			<sPassword>{$this->config['password']}</sPassword>
			<sType>affiliate</sType>
		</ns1:UserAuthentication>
		<ns1:getQuota SOAP-ENV:mustUnderstand="1" SOAP-ENV:actor="http://api.productserve.com/">true</ns1:getQuota>
	</SOAP-ENV:Header>
	<SOAP-ENV:Body>
		<ns1:getMerchant>
			<iMerchantId>$merchant_id</iMerchantId>
		</ns1:getMerchant>
	</SOAP-ENV:Body>
</SOAP-ENV:Envelope>
EOREQUEST;

		$request = <<<EOREQUEST
<?xml version="1.0" encoding="UTF-8"?>
<SOAP-ENV:Envelope xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/" xmlns:ns1="http://v3.core.com.productserve.com/" xmlns:ns2="http://api.productserve.com/">

	<SOAP-ENV:Header>
		<ns2:UserAuthentication SOAP-ENV:mustUnderstand="1" SOAP-ENV:actor="http://api.productserve.com/">
			<sApiKey>{$this->config['password']}</sApiKey>
		</ns2:UserAuthentication>
	</SOAP-ENV:Header>
	<SOAP-ENV:Body>
		<ns1:getMerchant>
			<iMerchantId>$merchant_id</iMerchantId>
			<sColumnToReturn>sLogoUrl</sColumnToReturn>			
			<sColumnToReturn>sDescription</sColumnToReturn>			
		</ns1:getMerchant>
	</SOAP-ENV:Body>
</SOAP-ENV:Envelope>
EOREQUEST;

		$response = $this->curl_file_get_contents($request);

		$xml = simplexml_load_string($response);
		$merchants = $xml->xpath('//ns1:getMerchantResponse/oMerchant');
		return $merchants[0];
	}
}

?>