<?php 

global $INSTALL_FOLDER;

require_once 'product_api.class.php';
require_once "$INSTALL_FOLDER/admin/includes/amazon.inc";

class Amazon extends ProductAPI {

	var $config = null;
	
	function Amazon($config) {
		$this->config = $config;
		
		//print_r($config);
	}
	
	function getKey() {
		return $this->config['api_key'];
	}
		
	function generateProductKey($product) {
		return $this->getKey() . '_' . $product->ASIN;			
	}
	
	function getDisplayPrice($product) {
		$display_price = ($product->Offers->Offer->OfferListing->Price->Amount);
		
		if ($display_price == '') {
			$display_price = ($product->ItemAttributes->ListPrice->Amount);		
		}
		
		if ($display_price == '') {
			$display_price = ($product->OfferSummary->LowestNewPrice->Amount);		
		}

		if (! $display_price) {
			$display_price = ($product->VariationSummary->LowestPrice->Amount);		
			
		}

		$display_price = $display_price / 100;		
		$display_price = sprintf("%01.2f", $display_price);
		return $display_price;
	}
	
	function getUsedPrice($product) {
		return ($product->OfferSummary->LowestUsedPrice->Amount) / 100;
	}
	
	function makePopulatedDSProduct($product) {		
		$p = array();
		$p['mykey'] 							= $this->generateProductKey($product);
		$p['network_product_key']	= $product->ASIN;
		$p['name'] 								= $product->ItemAttributes->Title;
		
		if ($product->EditorialReviews->EditorialReview->Content) {
			$p['description'] 				= $product->EditorialReviews->EditorialReview->Content;					
		} else {
			$p['description'] 				= $product->ItemAttributes->Title;								
		}
		
		$p['display_price'] 			= $this->getDisplayPrice($product);
		$p['deep_link']						= $product->DetailPageURL;  
		$p['thumb_url']						= $product->SmallImage->URL;  
		$p['image_url']						= $product->LargeImage->URL;  
		$p['datafeed_id'] 				= $this->config['datafeed_id'];
		$p['merchant_name'] 			= 'Amazon';
		$p['ws_api_key']					= $this->getKey();
		$p['used_price']					= $this->getUsedPrice($product);
		$p['currency_prefix']     = $this->config['currency_symbol'];
		return $p;
	}
	
	public function findProducts($options) {
		$products = array();

		if ($this->config['max_results']) {
			$maxPages = $this->config['max_results'] / 10;
		} else {
			$maxPages = 1;
		}
		
		$requestPage = 1;
		do {			
			$xml = ItemSearch($this->config, $options['category_id'], $options['keywords'], $requestPage++);

			foreach($xml->Items->Item as $product) {
				$products[] = $this->makePopulatedDSProduct($product);
			}
		} while (($requestPage <= $xml->Items->TotalPages) and ($requestPage <= $maxPages));

		return $products;
	}

	public function getCategories($options) {
		$categories = array();
		$categories[] = array('id' => 'Blended', 						'name' => 'ALL (Blended)');
		$categories[] = array('id' => 'Apparel', 						'name' => 'Apparel');
		$categories[] = array('id' => 'Automotive', 				'name' => 'Automotive');
		$categories[] = array('id' => 'Baby', 							'name' => 'Baby');
		$categories[] = array('id' => 'Beauty', 						'name' => 'Beauty');
		$categories[] = array('id' => 'Books', 							'name' => 'Books');
		$categories[] = array('id' => 'Clasical', 					'name' => 'Classical');
		$categories[] = array('id' => 'DVD', 								'name' => 'DVD');
		$categories[] = array('id' => 'Electronics', 				'name' => 'Electronics');
		$categories[] = array('id' => 'Grocery', 						'name' => 'Grocery');	
		$categories[] = array('id' => 'HealthPersonalCare', 'name' => 'Health and Personal Care');
		$categories[] = array('id' => 'HomeGarden', 				'name' => 'Home and Garden');		
		$categories[] = array('id' => 'Jewelry', 						'name' => 'Jewelry');		
		$categories[] = array('id' => 'Kitchen', 						'name' => 'Kitchen ');
		$categories[] = array('id' => 'Lighting', 					'name' => 'Lighting');		
		$categories[] = array('id' => 'MP3Downloads', 			'name' => 'MP3 Downloads');		
		$categories[] = array('id' => 'Music', 							'name' => 'Music');
		$categories[] = array('id' => 'MusicTracks', 				'name' => 'Music Tracks');		
		$categories[] = array('id' => 'OfficeProducts', 		'name' => 'Office Products');		
		$categories[] = array('id' => 'OutdoorLiving', 			'name' => 'Outdoor Living');		
		$categories[] = array('id' => 'Outlet', 						'name' => 'Outlet');		
		$categories[] = array('id' => 'Shoes', 							'name' => 'Shoes');
		$categories[] = array('id' => 'Software', 					'name' => 'Software');		
		$categories[] = array('id' => 'SoftwareVideoGames',	'name' => 'Software Video Games');				
		$categories[] = array('id' => 'Toys', 							'name' => 'Toys');
		$categories[] = array('id' => 'VHS', 								'name' => 'VHS');		
		$categories[] = array('id' => 'Video', 							'name' => 'Video');		
		$categories[] = array('id' => 'VideoGames', 				'name' => 'Video Games');
		$categories[] = array('id' => 'Watches',		 			'name' => 'Watches');		
		
		return $categories;
	}

	public function getMerchants($options) {
		
	}
	
	public function getProduct($options) {
		$xml = ItemLookup($this->config, $options['product_key']);
		$product = $xml->Items->Item;
		return $this->makePopulatedDSProduct($product);
	}
	
	public function isPaginationSupported() {
		return true;
	}
}

?>
