<?php 

require_once 'product_api.class.php';
require_once '../models/country_m.inc';

class CJ extends ProductAPI {

	var $config = null;
	
	function CJ($config) {
		$this->config = $config;
	}
	
	function getKey() {
		return "CJ";
	}
		
	function generateAPIKey($product) {
		return $this->getKey() . '_' . $product->{'advertiser-id'};	
	}
		
	function generateProductKey($product) {
		return $this->getKey() . '_' . $product->{'ad-id'} . '-' . $product->{'sku'};			
	}
		
	function curl_file_get_contents($options) {

		$ch = curl_init();
		$timeout = 60; // set to zero for no timeout
		
		$keywords = urlencode($options['keywords']);
		
		curl_setopt ($ch, CURLOPT_URL, "https://product-search.api.cj.com/v2/product-search?website-id={$this->config['website_id']}&keywords=$keywords&currency={$this->config['currency']}");
		curl_setopt ($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt ($ch, CURLOPT_CONNECTTIMEOUT, $timeout);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);		
		curl_setopt ($ch, CURLOPT_HTTPHEADER, array('Authorization: ' . $this->config['developer_key']));

		$response = curl_exec($ch);
		curl_close($ch);
		//echo("<textarea>$response</textarea>");
		return $response;
	}
		
	
	function makePopulatedDSProduct($product) {
		
		$p = array();
		$p['mykey'] 							= $this->generateProductKey($product);
		$p['network_product_key']	= $product->name  . '|' . $product->{'ad-id'} . '-' . $product->{'sku'};
		$p['name'] 								= (String) $product->name;
		$p['description'] 				= (String) $product->description;
		$p['display_price'] 			= (String) $product->price;
		$p['deep_link']						= (String) $product->{'buy-url'};  
		$p['thumb_url']						= (String) $product->{'image-url'};  
		$p['image_url']						= (String) $product->{'image-url'};  
		$p['ws_api_key']					= $this->getKey();
		$country = find_country_by_currency_code((String) $product->{'currency'});
		$p['currency_prefix']			= $country['currency_symbol'];
		
		$datafeed = datafeed_findby_apikey($this->generateAPIKey($product));
		
		if ($datafeed) {
			$p['datafeed_id'] = $datafeed['id'];
		} else {
			$datafeed = array();			
			$datafeed['merchant_name']  = (String) $product->{'advertiser-name'};
			$datafeed['merchant_logo']  = "nologo.gif";
			$datafeed['parser_id'] 		  = -1;
			$datafeed['description'] 	  = ''; 
			$datafeed['datafile']			  = '';
			$datafeed['url']					  = '';
			$datafeed['feed_type']	 	  = 'webservice';
			$datafeed['status']					= 'N/A';
			$datafeed['total_products'] = -1;
			$datafeed['api_key']				= $this->generateAPIKey($product);
			$datafeed['parser_id']			= -1;
			$datafeed['is_suspended']		= 0;
			

			if ($country['iso3']) {
				$datafeed['country_code']  = $country['iso3'];				
			} else {
				$datafeed['country_code']  = null;
			}
			
			datafeed_insert($datafeed);
		}

		$p['merchant_name'] = (String) $product->{'advertiser-name'};
		return $p;
	}
	
	public function findProducts($options) {
		$products = array();
		$response = $this->curl_file_get_contents($options);
		$xml = simplexml_load_string($response);

		foreach($xml->xpath('//cj-api/products/product') as $product) {
			$products[] = $this->makePopulatedDSProduct($product);
		}

		return $products;
	}
	
	public function getCategories($options) {
		$categories = array();

		$cat = array();
		$cat['id'] = '-1';
		$cat['name'] = 'ALL';
		$categories[] = $cat;


		return $categories;

	}

	public function getMerchants($options) {
		
	}
	
	public function getProduct($options) {
		$key = $options['product_key'];
		
		list($name, $adId) = split('\|', $key);

		$products = $this->findProducts(array('keywords' => $name));
		foreach ($products as $product) {			
			if ($product['network_product_key'] == $key) {
				return $product;
			}
		}		
	}	
	
	public function getMessage() {
		return "<strong>Note : </strong> Merchants on the CJ network do <strong>NOT</strong> automatically have logos created. If you haven't done so previously you will need to supply these on the 'Edit Datafeed' screen for the respective merchant.";
	}
}

?>