<?php 

require_once 'product_api.class.php';
require_once '../models/country_m.inc';

class LinkShare extends ProductAPI {

	var $config = null;
	
	function LinkShare($config) {
		$this->config = $config;
	}
	
	function getKey() {
		return "LS";
	}
	
	function generateAPIKey($product) {
		return $this->getKey() . '_' . $product->mid;	
	}
	
	function generateProductKey($product) {
		return $this->getKey() . '_' . $product->linkid;			
	}
	
	function curl_file_get_contents($request) {

		//echo("<textarea>$request</textarea><br/>");

		$ch = curl_init();
		$timeout = 60; // set to zero for no timeout
		curl_setopt ($ch, CURLOPT_URL, $request);		
		curl_setopt ($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt ($ch, CURLOPT_CONNECTTIMEOUT, $timeout);

		$response = curl_exec($ch);
		curl_close($ch);
		
		//echo("<textarea>$response</textarea>");
		return $response;
		
	}

	function makePopulatedDSProduct($product) {
		$p = array();
		$p['mykey'] 							= $this->generateProductKey($product);
		$p['network_product_key']	= $product->productname . '|' . $product->mid;
		$p['name'] 								= $product->productname;
		$p['description'] 				= $product->description . $product->description->long;
		$p['display_price'] 			= $product->price;
		$p['deep_link']						= $product->linkurl;  
		$p['thumb_url']						= $product->imageurl;  
		$p['image_url']						= $product->imageurl;  
		$p['ws_api_key']					= $this->getKey();
		$country = find_country_by_currency_code($product->price['currency']);
		$p['currency_prefix']			= $country['currency_symbol'];

		$datafeed = datafeed_findby_apikey($this->generateAPIKey($product));
		
		if ($datafeed) {
			$p['datafeed_id'] = $datafeed['id'];
		} else {

			$datafeed = array();
			
			$datafeed['merchant_name']  = $product->merchantname;
			$datafeed['merchant_logo']  = '';
			$datafeed['parser_id'] 		  = -1;
			$datafeed['description'] 	  = ''; 
			$datafeed['datafile']			  = '';
			$datafeed['url']					  = '';
			$datafeed['feed_type']	 	  = 'webservice';
			$datafeed['status']					= 'N/A';
			$datafeed['total_products'] = -1;
			$datafeed['api_key']				  = $this->generateAPIKey($product);
			$datafeed['parser_id']			= -1;
			$datafeed['is_suspended']		= 0;

			if ($country['iso3']) {
				$datafeed['country_code']  = $country['iso3'];				
			} else {
				$datafeed['country_code']  = null;
			}

			datafeed_insert($datafeed);
		}

		$p['merchant_name'] = $datafeed['merchant_name'];
		return $p;
	}
	
	public function stripInvalidCharacters($keywords) {
		$keywords = str_replace(' ', '+', $keywords);
		$keywords = str_replace('-', '+', $keywords);
		$keywords = str_replace('[', '', $keywords);
		$keywords = str_replace(']', '', $keywords);
		$keywords = str_replace('(', '', $keywords);
		$keywords = str_replace(')', '', $keywords);
		$keywords = str_replace('{', '', $keywords);
		$keywords = str_replace('}', '', $keywords);
		$keywords = str_replace('<', ' ', $keywords);
		$keywords = str_replace('>', ' ', $keywords);
		$keywords = str_replace('"', '', $keywords);
		$keywords = str_replace(',', '', $keywords);
		return $keywords;
	}
	
	public function findProducts($options) {	
	$keywords = urlencode($options['keywords']);
	
	$max_results = $this->config['max_results'];
	if (! $max_results) {
		$max_results = 50;
	}
		
	$request = "http://productsearch.linksynergy.com/productsearch?token={$this->config['token']}&keyword=$keywords&cat=&MaxResults=$max_results&pagenumber=1&sort=productname&sorttype=asc";
	
		$products = array();
		$response = $this->curl_file_get_contents($request);
		$xml = simplexml_load_string($response);

		foreach($xml->xpath('//result/item') as $product) {
			$products[] = $this->makePopulatedDSProduct($product);
		}

		return $products;
	}
	
	public function getCategories($options) {
	
		$categories = array();
		$dummy = array('id' => '-1', 'name' => 'N/A');
		$categories[]= $dummy;
		return $categories;
	}

	public function getMerchants($options) {
		
	}
	
	public function getProduct($options) {
				
		$key = $options['product_key'];		
		list($keywords, $mid) = split('\|', $key);
		
		$keywords = $this->stripInvalidCharacters($keywords);		

		$request = "http://productsearch.linksynergy.com/productsearch?token={$this->config['token']}&keyword=\"$keywords\"&cat=&MaxResults=1&pagenumber=1&sort=productname&sorttype=asc&mid=$mid";
				
		//syslog(1, $request);
		
		$response = $this->curl_file_get_contents($request);
		$xml = simplexml_load_string($response);
		
		$products = $xml->xpath('//result/item');
		return $this->makePopulatedDSProduct($products[0]);
}

	
	public function getMerchant($merchant_id) {
		
	}

	public function getMessage() {
		return "<strong>Note : </strong> Merchants on the LinkShare network do <strong>NOT</strong> automatically have logos created. If you haven't done so previously you will need to supply these on the 'Edit Datafeed' screen for the respective merchant.";
	}
	
}

?>