<?php

function category_insert($category) {
	
	global $db;
	$statement = $db->prepare("insert into category(name, cat_type, blurb, description, sql_code, slug, meta_description, meta_keywords, nofollow, parent_category_id) values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
	$statement->bind_param('ssssssssii', 
		$category['name'],
		$category['cat_type'],
		$category['blurb'],
		$category['description'],
		$category['sql_code'],
		$category['slug'],
		$category['meta_description'],
		$category['meta_keywords'],
		$category['nofollow'],
		$category['parent_category_id']
	);

	$result = $statement->execute();	
	$statement->close();	
	return $db->insert_id;
}

function category_update($category) {
	global $db;
	$statement = $db->prepare("update category set name = ?, cat_type = ?, blurb = ?, description = ?, sql_code = ?, slug = ?, meta_description = ?, meta_keywords = ?, nofollow = ?, parent_category_id = ? where id = ?");
	$statement->bind_param('ssssssssiii', 
		$category['name'],
		$category['cat_type'],
		$category['blurb'],
		$category['description'],
		$category['sql_code'],
		$category['slug'],
		$category['meta_description'],
		$category['meta_keywords'],
		$category['nofollow'],
		$category['parent_category_id'],
		$category['id']
	);

	$result = $statement->execute();
	$statement->close();	
}

function category_find($id) {
	global $db;
	
	if (is_numeric($id)) {
		$query = "select * from category where id = $id";
	} else {
		$query = "select * from category where slug = '$id'";
	}
	
	$result = $db->query($query);
	return $result->fetch_array();
}

function category_find_all() {
	global $db;
	$query = "select * from category order by name";
	return $db->query($query);
}

function category_find_all_parents() {
	global $db;
	$query = "select * from category where (parent_category_id is null or parent_category_id < 1) order by name";
	return $db->query($query);
}

function category_find_all_auto_parents() {
	global $db;
	$query = "select * from category where (parent_category_id is null or parent_category_id < 1) and cat_type = 'auto' order by name";
	return $db->query($query);
}

function category_find_all_auto_children($parent_category_id) {
	global $db;
	$query = "select * from category where (parent_category_id = $parent_category_id) and cat_type = 'auto' order by name";
	return $db->query($query);
}

function category_find_children($parent_id) {
	global $db;
	$query = "select * from category where parent_category_id = $parent_id order by name";
	return $db->query($query);
}

function category_find_with_ids($catid_array) {
	global $db;
	$catid_array_csv = implode(",", $catid_array);
	
	$query = "select * from category where id in ($catid_array_csv) order by name";
	return $db->query($query);
}


function category_find_all_manual() {
	global $db;
	$query = "select * from category where cat_type = 'manual' order by name";
	return $db->query($query);
}

function category_delete($id) {
	global $db;
	$query = "delete from category_product where category_id = $id";
	$db->query($query);
	
	$query = "delete from category where id = $id";
	$db->query($query);
}

function category_add_products($category_id, $products) {
	global $db;
	
	foreach ($products as $mykey) {	
		$query = "insert into category_product(category_id, mykey) values (?, ?)";
		$statement = $db->prepare($query);
		$statement->bind_param('is', $category_id, $mykey);
		$result = $statement->execute();
		$statement->close();
	}	
}

function category_find_products($category) {
	global $db;
	
	if ($category['cat_type'] == 'manual') {
		$query = "select p.*, m.*, d.*, cp.* from category_product cp, datafeeds d, products as p left outer join meta as m on p.mykey = m.mykey where cp.mykey = p.mykey and p.datafeed_id = d.id and cp.category_id = {$category['id']}";
	} else if ($category['cat_type'] == 'auto') {
		if ($category['parent_category_id']) {
			$query = "select * from products p left outer join meta as m on p.mykey = m.mykey where sub_category_id = {$category['id']}";					
		} else {
			$query = "select * from products p left outer join meta as m on p.mykey = m.mykey where category_id = {$category['id']}";			
		}
		
		
	}
	else {
		$query = stripslashes($category['sql_code']);
	}

	return $db->query($query);	
}

function category_remove_products($category_id, $products) {
	global $db;

	foreach ($products as $mykey) {	
		$query = "delete from category_product where category_id = ? and mykey = ?";
		$statement = $db->prepare($query);
		$statement->bind_param('is', $category_id, $mykey);
		$result = $statement->execute();
		$statement->close();
	}	
}
?>
