<?php
/* 
Plugin Name: Datafeed Studio Plugin
Plugin URI: http://datafeedstudio.com 
Description: Easily embed Datafeed Studio products within your site.  
Version: 1.0 
Author: Martin Wood
Author URI: http://martinwood.org
*/

$ds_path = get_option('ds_path');

if ($ds_path) {
	include "$ds_path/admin/includes/config.inc";	
	include "$ds_path/admin/includes/amazon.inc";
	include "$ds_path/lib/database.inc";
	include "$ds_path/lib/functions.inc";
}

function dstudio_pricebox($options = array()) {
			
	global $post;
	global $db;
	global $CURRENCY_PREFIX;
	
	if ($options['query']) {
		$ds_query = $options['query'];
	}	else {
		$ds_query = get_post_meta($post->ID, 'ds_query', true);		
	}
			
	if (! ($ds_query)) {
		return;
	}
	
	if ($options['type']) {
		$ds_type = $options['type'];
	} else {
		$ds_type = 'classic';
	}
	
	$ds_path = get_option('ds_path');
	$ds_url = get_option('ds_url');
	$ds_redirect_url = get_option('ds_redirect_url');
	$ds_columns = get_option('ds_columns');
	$ds_heading = get_option('ds_heading');
	
	if (! $options['title']) {
		$ds_title = get_post_meta($post->ID, 'ds_title', true);
	} else {
		$ds_title = $options['title'];
	}
			
	$products = get_products($db, stripslashes($ds_query), '' , 1, $ds_title, get_option('ds_clickref') . $options['clickref']);
	if (count($products) == 0) {
		return;
	}
	
	$i = 0;
	$updated_products = array();
	foreach ($products as $product) {
		$i++;
		$row_style = "";
		if (($i % 2) == 0) {
		 $row_style = "alt";
		}

		if (str_startswith($product['deep_link'], 'http')) {
		  $deep_link = $product['deep_link'];
		} else {
		
			if (! $ds_redirect_url) {
				$ds_redirect_url = $ds_url;
			}
		
		  $deep_link = $ds_redirect_url . '/' . $product['deep_link'];
		}

		$product['deep_link'] = $deep_link;
		
		$updated_products []= $product;
	}

	$render_function = "render_$ds_type";
	$render_function($updated_products, $ds_title, $ds_columns, $ds_url, $ds_heading);

}

function render_right($products, $ds_title, $ds_columns, $ds_url, $ds_heading) {
	global $CURRENCY_PREFIX;
	
	foreach ($products as $product) {
		$rows .=<<<EOROW
<a rel="nofollow" target="_blank"><img style="margin: auto" width="100" src="{$product['image_url']}"/></a><br/>
<a rel="nofollow" target="_blank" href="{$product['deep_link']}">{$product['name']}</a><br/>
<p style="text-align: left"><a rel="nofollow" target="_blank" href="{$product['deep_link']}">$CURRENCY_PREFIX{$product['display_price']} from {$product['merchant_name']}</a></p>
<hr style=""/>
EOROW;
	}
	
	echo <<<EOHTML
<div style="clear: both; float: right; border-style: solid; width: 150px; padding: 0px 5px 0px 5px; border-color: orange; border-width: 1px; margin-left: 20px">
$rows
</div>
EOHTML;
}

function render_classic($products, $ds_title, $ds_columns, $ds_url, $ds_heading) {
	global $CURRENCY_PREFIX;

	if ($ds_title) {
		$ds_heading = str_replace('%title%', $ds_title, $ds_heading);
	
		if ($ds_heading) {
			$permalink = get_permalink();
		  echo("<h2><a href=\"$permalink\">");
		  echo $ds_heading;
		  echo('</a></h2>');
		}
	}	

	echo('<a name="prices"></a>');
  echo('<div style="clear:both"><table class="pricebox">');

	if ($ds_columns) {
		echo "<thead><tr>";
		$columns = split(",", $ds_columns);

			foreach ($columns as $column) {
				echo <<<EOHTML
<th>$column</th>
EOHTML;
			}	

			echo "</thead></tr>";

	}

	foreach ($products as $product) {
		
		if ($product['used_price'] > 0) {
			$used_price =<<<USED_PRICE
			<p style="font-size: smaller">Used : $CURRENCY_PREFIX{$product['used_price']}</p>
USED_PRICE;
		} else {
			$used_price = '';
		}
		
		
		
		echo <<<EOHTML
		<tr class="$row_style">
			<td>
				<a rel="nofollow" target="_{$product['mykey']}" href="{$product['deep_link']}"><img width="100" src="{$product['image_url']}"} border="0" align="left"/></a>
			</td>

			<td>{$product['name']} (<a rel="nofollow" target="_{$product['mykey']}" href="{$product['deep_link']}">{$product['merchant_name']}</a>)</td>

			<td class="pricecol" align="center">
				<a rel="nofollow" target="_{$product['mykey']}" href="{$product['deep_link']}">
				<img src="{$product['merchant_logo']}" height="31" width="88" alt="{$product['merchant_name']} Logo" border="0"/>
				</a>
				<br/><br/>
				<a target="_{$product['mykey']}" rel="nofollow" href="{$product['deep_link']}"><strong>$CURRENCY_PREFIX{$product['display_price']}</strong>$used_price</a>
				<br/><br/>
				<a rel="nofollow" target="_{$product['mykey']}" href="{$product['deep_link']}"><img border="0" src="$ds_url/dsimages/buynow.gif" alt="Buy Now"/>
			</td>
		</tr>
EOHTML;
	}

	echo('</table></div>');
}

add_action('admin_menu', 'dstudio_plugin_menu');


function dstudio($atts, $content = null) {

	ob_start();
	
	dstudio_pricebox($atts);
	$content = ob_get_contents();
	ob_end_clean();
	return $content;
}

add_shortcode('ds', 'dstudio');

function dstudio_plugin_menu() {
  add_options_page('Datafeed Studio Options', 'Datafeed Studio', 8, __FILE__, 'dstudio_options');
}

function dstudio_options() {	
?>
		<div class="wrap">
		<h2>Datafeed Studio Plugin Configuration</h2>

		<form method="post" action="options.php">
		<?php wp_nonce_field('update-options'); ?>

		<table class="form-table">

		<tr valign="top">
		<th scope="row"><strong>Path to Datafeed Studio</strong> on this host (e.g. /home/www/dstudio)</th>
		<td><input type="text" size="50" name="ds_path" value="<?php echo get_option('ds_path'); ?>" /></td>
		</tr>

		<tr valign="top">
		<th scope="row"><strong>URL</strong> of your main Datafeed Studio installation (e.g. http://example.com)</th>
		<td><input type="text" size="50" name="ds_url" value="<?php echo get_option('ds_url'); ?>" /></td>
		</tr>

		<tr valign="top">
		<th scope="row"><strong>Redirect URL</strong> (Advanced Use Only). If set, the plugin will use this URL as the basis of the deep links. This allows you to provide your own script to handle all clicks, otherwise leave blank.</th>
		<td><input type="text" size="50" name="ds_redirect_url" value="<?php echo get_option('ds_redirect_url'); ?>" /></td>
		</tr>

		<tr valign="top">
		<th scope="row"><strong>Click Reference</strong>. Pass this value with any product clicks for affiliate order tracking.</th>
		<td><input type="text" size="50" name="ds_clickref" value="<?php echo get_option('ds_clickref'); ?>" /></td>
		</tr>


		<tr valign="top">
		<th scope="row"><strong>Columns (optional)</strong>. Comma-separated list of column names to display above the price compare box.</th>
		<td><input type="text" size="50" name="ds_columns" value="<?php echo get_option('ds_columns'); ?>" /></td>
		</tr>

		<tr valign="top">
		<th scope="row"><strong>Heading</strong>. Optional heading that appears above a price compare box. Just enter a space for no title, or enter your preferred heading text. To include the title of the product enter %title%, e.g. Best Price for %title%</th>
		<td><input type="text" size="50" name="ds_heading" value="<?php echo get_option('ds_heading'); ?>" /></td>
		</tr>



		</table>

		<input type="hidden" name="action" value="update" />
		<input type="hidden" name="page_options" value="ds_path,ds_url,ds_redirect_url,ds_clickref,ds_columns,ds_heading" />

		<p class="submit">
		<input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
		</p>

		</form>
		</div>

<?php
}

?>