<?php
/* 
Plugin Name: Datafeed Studio Plugin
Plugin URI: http://datafeedstudio.com 
Description: Easily embed Datafeed Studio products within your site.  
Version: 1.0 
Author: Martin Wood
Author URI: http://martinwood.org
*/

function dstudio_slug_clickref($str) {
	$str = strtolower(trim($str));
	$str = preg_replace('/[^a-z0-9-_]/', '-', $str);
	$str = preg_replace('/-+/', "-", $str);
	return $str;
}

function dstudio_select($current_option, $check_against_option) {
	if ($current_option == $check_against_option) {
		echo('selected');
	}
}

function dstudio_check_cache($cache_folder, $cachefile, $cachetime, $filetype = 'html') {

	$cachefile = "$cache_folder/$cachefile" ;

	if (file_exists($cachefile) && (time() - $cachetime < filemtime($cachefile)))  {		
		
		$data = file_get_contents($cachefile);
		
		if ($filetype == 'html') {
   		$data .= "<!-- DS:Cached ".date('jS F Y H:i', filemtime($cachefile))." -->\n";			
		}
		
		echo $data;
		return true;
	}

	ob_start();
	return false;
}

function dstudio_write_to_cache($cache_folder, $cachefile) {
	if (! file_exists($cache_folder)) {
		echo("<!-- DS:No cache folder-->");
		return;
	}
	
	if ( ! dstudio_get_cache_seconds()) {
		echo("<!-- DS:No cache seconds -->");
		return;
	}
	
	$cachefile = "$cache_folder/$cachefile";				
	$fp = fopen("$cachefile", 'w'); 
	fwrite($fp, ob_get_contents()); 
	fclose($fp); 
	ob_end_flush(); 
}

function dstudio_empty_cache($cache_folder) {
	foreach(glob("$cache_folder/*.dscache") as $cache_file) {
	    unlink($cache_file);
	}
}

function dstudio_generate_cache_name($dstudio_query) {
	return  $_SERVER['SERVER_NAME'] . '_'. md5($dstudio_query) . '.dscache';
}

function dstudio_get_cache_seconds() {
	return get_option('dstudio_cache_seconds');
}

function dstudio_str_startswith($source, $prefix) {
   return strncmp($source, $prefix, strlen($prefix)) == 0;
}

function dstudio_http_get_request($url) {
	$ch = curl_init();
	$timeout = 60; // set to zero for no timeout
	curl_setopt($ch, CURLOPT_URL, $url);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, $timeout);
	curl_setopt($ch, CURLOPT_REFERER, 'http://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']);

	$response = curl_exec($ch);
	curl_close($ch);
	return $response;
}

function dstudio_make_sort_option($ds_sort) {
	
	if (! $ds_sort) {
		$ds_sort = 'phtl';
	}
	switch($ds_sort) {
		case 'mnazhigh':
			return " order_field=d.merchant_name,p.display_price order_direction=desc";
			break;
		case 'mnazlow':
			return " order_field=d.merchant_name,p.display_price order_direction=asc";
			break;
		case 'phtl':
			return " order_field=p.display_price order_direction=desc";
			break;
		case 'plth':
			return " order_field=p.display_price order_direction=asc";
			break;
		default:
			echo "<strong>DS Plugin Error : Unexpected sort option '$ds_sort'</strong><br/>";
	}
}

function dstudio_get_country_code() {
	$ip = $_SERVER['REMOTE_ADDR'];
	$host = "http://www.geoplugin.net/php.gp?ip=$ip";
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, $host);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	$response = curl_exec($ch);
	curl_close ($ch);

	$data = unserialize($response);
	$code = $data['geoplugin_countryCode'];		
	//$code = 'GB';
	return $code;	
}

function dstudio_pricebox($options = array()) {
			
	global $post;
	global $db;
		
	if ($options['query']) {
		$ds_query = $options['query'];
	}	else {
		$ds_query = get_post_meta($post->ID, 'ds_query', true);		
	}
			
	if (! ($ds_query)) {
		return;
	}

	if (! dstudio_str_startswith(strtolower($ds_query), 'select ')) {
		if (! (strpos($ds_query, 'order_direction') || (strpos($ds_query, 'order_field')) || (strpos($ds_query, 'order by')))) {
			if (! strpos($ds_query, '|')) {
				$ds_query = $ds_query . ' | ';
			}

			$ds_query .= dstudio_make_sort_option(get_option('ds_sort'));
		}

		$dstudio_geo_enabled = get_option('dstudio_geo_enabled');
		if ($dstudio_geo_enabled) {
			if (! (strpos($ds_query, 'country_code'))) {
		
				if (! strpos($ds_query, '|')) {
					$ds_query = $ds_query . ' | ';
				}

				$country_code = dstudio_get_country_code();
				$ds_query .=  ' country_code=' . $country_code;
			}
		}	else {
			$country_code = '';
		}
	}
	
	$ds_url = get_option('ds_url');
	$ds_redirect_url = get_option('ds_redirect_url');
	$ds_heading = get_option('ds_heading');
	$pricebox = get_option('ds_pricebox');
	
	if (! $options['title']) {
		$ds_title = get_post_meta($post->ID, 'ds_title', true);
	} else {
		$ds_title = $options['title'];
	}

	list($ds_lhs, $ds_rhs) = explode('|', $ds_query);
	$ds_query = "$ds_lhs " . get_option("ds_xquery") . " | $ds_rhs " . get_option("ds_xoption");
			
	$query = urlencode(stripslashes($ds_query));
	$title = urlencode(stripslashes($ds_title));

	$clickref = get_option('ds_clickref') . $options['clickref'];

	if ($options['pricebox']) {
		$pricebox = $options['pricebox'];
	} elseif (get_post_meta($post->ID, 'ds_pricebox', true)) {
		$pricebox = get_post_meta($post->ID, 'ds_pricebox', true);	
	} elseif (get_option('ds_pricebox')) {
		$pricebox = get_option('ds_pricebox');
	} else {
		$pricebox = 'default';
	}
	
	$pricebox = urlencode($pricebox);
	
	if (in_array('ignorezero', $options)) {
		$ignorezero = urlencode('&ignorezero=1');
	} else {
		$ignorezero = '';
	}
			
	$url = "$ds_url/api/v1/products?search=$query&clickref=$clickref&pricebox=$pricebox&redirect_url=$ds_redirect_url$ignorezero";
	
	if (dstudio_check_cache(get_option('dstudio_cache_folder'), dstudio_generate_cache_name($url), dstudio_get_cache_seconds(), 'html')) {
		return;
	}

	$response = dstudio_http_get_request($url);
	
	//echo("<textarea>$response</textarea>");
	
	if ($ds_title) {
		$ds_heading = str_replace('%title%', $ds_title, $ds_heading);		
	}

	if ($ds_heading) {
		echo $ds_heading;
	}
	
	echo $response;
	dstudio_write_to_cache(get_option('dstudio_cache_folder'), dstudio_generate_cache_name($url));

}

add_action('admin_menu', 'dstudio_plugin_menu');


function dstudio($atts, $content = null) {
	ob_start();
	dstudio_pricebox($atts);
	$content = ob_get_contents();
	ob_end_clean();
	return $content;
}

function dstudio_search($atts, $content = null) {
	ob_start();	
	dstudio_searchbox_form($atts);
	$content = ob_get_contents();
	ob_end_clean();
	return $content;
}

function dstudio_search_result($atts, $content = null) {
	ob_start();	
	dstudio_searchbox_result($atts);
	$content = ob_get_contents();
	ob_end_clean();
	return $content;
}

function dstudio_searchbox_form($options = null) {
		echo <<<EOHTML
<div id='dstudio_searchbox_form'>
<form method='post'>
  <input type='text' name='query'/>
  <input type='submit'/>
</form>
</div>
EOHTML;
}

function dstudio_searchbox_result($options = null) {
	if (($_POST['query']) && (strlen($_POST['query']) >= 3)) {
		$query = $_POST['query'];
		$query = str_replace(' ', ' +', $query);
		$query = '+' . $query;
		
		$query .= ' | max_results=50';
		
		$options['query'] = $query;
		dstudio_pricebox($options);
	}	
}

add_shortcode('ds', 'dstudio');
add_shortcode('ds_search', 'dstudio_search');
add_shortcode('ds_search_result', 'dstudio_search_result');


function dstudio_plugin_menu() {
  add_options_page('Datafeed Studio Options', 'Datafeed Studio', 8, __FILE__, 'dstudio_options');
}

function dstudio_options() {		
	
	$dstudio_cache_folder = get_option('dstudio_cache_folder');
	if (! $dstudio_cache_folder) {
		$dstudio_cache_folder = '/tmp';
	}
	
	if (! is_writable($dstudio_cache_folder)) {
		$cache_message = "<span style='color: red'><b>Error</b> - I can't write to the folder '$dstudio_cache_folder'. Make sure the folder exists and is writable.</span>";
	} else {
		$cache_message = "";
	}
	
	$dstudio_cache_seconds = get_option('dstudio_cache_seconds');
	
	if (($_GET['empty_cache']) && (is_writable($dstudio_cache_folder))) {
		dstudio_empty_cache($dstudio_cache_folder);
		$cache_message = "<span style='color: green'><b>Cache Emptied</b></span>";
	}
	
	$dstudio_geo_enabled = get_option('dstudio_geo_enabled');
	if ($dstudio_geo_enabled) {
		$dstudio_geo_checked = 'checked';
	} else {
		$dstudio_get_checked = '';
	}
	
	$dstudio_clickref = dstudio_slug_clickref(get_option('ds_clickref'));
?>
		<div class="wrap">
		<h2>Datafeed Studio Plugin Configuration</h2>

		<form method="post" action="options.php">
		<?php wp_nonce_field('update-options'); ?>

		<table class="form-table">

		<tr valign="top">
		<th scope="row"><strong>URL</strong> of your main Datafeed Studio installation (e.g. http://example.com)</th>
		<td><input type="text" size="50" name="ds_url" value="<?php echo get_option('ds_url'); ?>" /></td>
		</tr>

		<tr valign="top">
		<th scope="row"><strong>Redirect URL</strong> (Advanced Use Only). If set, the plugin will use this URL as the basis of the deep links. This allows you to provide your own script to handle all clicks, otherwise leave blank.</th>
		<td><input type="text" size="50" name="ds_redirect_url" value="<?php echo get_option('ds_redirect_url'); ?>" /></td>
		</tr>

		<tr valign="top">
		<th scope="row"><strong>Click Reference</strong>. Pass this value with any product clicks for affiliate order tracking.</th>
		<td><input type="text" size="50" name="ds_clickref" value="<?php echo $dstudio_clickref ?>" /></td>
		</tr>

		<tr valign="top">
		<th scope="row"><strong>Extra Query</strong>. (Advanced) Add the contents of this string to <b>ALL</b> queries, e.g. use '+widget' to always add the word 'widget' to queries.</th>
		<td><input type="text" size="50" name="ds_xquery" value="<?php echo get_option('ds_xquery'); ?>" /></td>
		</tr>

		<tr valign="top">
		<th scope="row"><strong>Extra Option</strong>. (Advanced) Add the contents of this string to <b>ALL</b> queries as 'extra options' - , e.g. use 'min_price=50' to only include products more than 50. See <a target="_blank" href="<?php echo get_option('ds_url'); ?>/admin/views/help.php?topic=pricecomparesyntax">shortcode syntax</a> for more info.</th>
		<td><input type="text" size="50" name="ds_xoption" value="<?php echo get_option('ds_xoption'); ?>" /></td>
		</tr>


		<tr valign="top">
		<th scope="row"><strong>Heading</strong>. Optional HTML that appears above a price compare box. Just enter a space for no title, or enter your preferred heading text. To include the title of the product enter %title%, e.g. Best Price for %title%</th>
		<td><textarea name="ds_heading" rows="4" cols="40"><?php echo get_option('ds_heading'); ?></textarea></td>
		</tr>

			<?php
		$ds_url = get_option('ds_url');
		$response = dstudio_http_get_request("$ds_url/api/v1/priceboxes?" . time());
		
		libxml_use_internal_errors(true);
		$xml = simplexml_load_string($response);

		if ($xml) {

			echo <<<EOHTML
			<tr valign="top">
			<th scope="row"><strong>Price Box Name</strong>. Enter the name of the Price Box style (as defined at your DS admin site) you would like to use or leave blank for the default style.</th>
			<td>
			<select name='ds_pricebox'>
EOHTML;


					$priceboxes = $xml->priceboxes;

					foreach ($priceboxes->pricebox as $pricebox) {
						if ($pricebox == get_option('ds_pricebox')) {
							$selected_pricebox = 'selected';
						} else {
							$selected_pricebox = '';
						}

						echo <<<EOPBOX
			<option value='$pricebox' $selected_pricebox>$pricebox</option>
EOPBOX;

					}

			echo <<<EOHTML
						</select>
						</td>
						</tr>

EOHTML;
			
		} else {
			echo <<<EOPBOX
<input type='hidden' name='ds_pricebox' value='default'/>
EOPBOX;

		}
		?>			

			<tr valign="top">
				<th scope="row"><strong>Sort Order</strong>&nbsp;Select the default ordering system for your results (note that this can be overridden on a per price box basis using <a target="_blank" href="<?php echo get_option('ds_url'); ?>/admin/views/help.php?topic=pricecomparesyntax">shortcode syntax</a>)</th>
				<td>
				 <select name='ds_sort'>
					<option value='mnazhigh' <?php dstudio_select(get_option('ds_sort'), 'mnazhigh'); ?>>Merchant Name A-Z (with the highest price listed first)</option>
					<option value='mnazlow' <?php dstudio_select(get_option('ds_sort'), 'mnazlow'); ?>>Merchant Name A-Z (with the lowest price listed first)</option>
					<option value='phtl' <?php dstudio_select(get_option('ds_sort'), 'phtl'); ?>>Prices High to Low</option>
					<option value='plth' <?php dstudio_select(get_option('ds_sort'), 'plth'); ?>>Prices Low to High</option>
				 </select>
				</td>
			</tr>



			<tr valign="top">
				<th scope="row"><strong>Cache Folder</strong>. This plugin will cache results from your Datafeed Studio server inside this folder. Make sure <strong>it exists</strong> and <strong>is writable</strong> by the user your web server runs as.</th>
				<td><input type="text" size="60" name="dstudio_cache_folder" value="<?php echo $dstudio_cache_folder ?>"/>
					<a href="<?= $_SERVER['REQUEST_URI'] ?>&amp;empty_cache=true">Empty Cache!</a>
					<br/><?php echo $cache_message ?> 
				</td>
			</tr>

				<tr valign="top">
					<th scope="row"><strong>Cache Time in Seconds</strong>. Number of <b>seconds</b> to cache results for locally. Leave blank or set to zero for no caching.</th>
					<td><input type="text" size="10" name="dstudio_cache_seconds" value="<?php echo $dstudio_cache_seconds ?>" /><br/></td>
				</tr>

				<tr valign="top">
					<th scope="row"><strong>Enable Geolocation?</strong> (Advanced) If enabled the plugin will automatically add a country code to any queries based on the IP address of the visitor. Only useful if your server has feeds from multiple countries. Geolocation must also be enabled on your DS server.</th>
					<td><input type="checkbox" name="dstudio_geo_enabled" value="1" <?php echo $dstudio_geo_checked ?>/><br/></td>
				</tr>

		</table>

		<input type="hidden" name="action" value="update" />
		<input type="hidden" name="page_options" value="ds_url,ds_redirect_url,ds_clickref,ds_xoption,ds_xquery,ds_heading,ds_pricebox,dstudio_cache_folder,dstudio_cache_seconds,ds_sort,dstudio_geo_enabled" />

		<p class="submit">
		<input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
		</p>

		</form>
		</div>

<?php
}

?>
