<?php			

function slug_func($str) {
	$str = strtolower(trim($str));
	$str = preg_replace('/[^a-z0-9-]/', '-', $str);
	$str = preg_replace('/-+/', "-", $str);
	return $str;
}

function generate_product_url($mykey, $userslug = '', $autoslug = '') {
	global $SEO_FRIENDLY_URLS;
	global $AUTOSLUG_ENABLED;
			
	if ($SEO_FRIENDLY_URLS) {
		if ($userslug) {
			$mykey = $userslug;
		} elseif (($autoslug != '') && ($AUTOSLUG_ENABLED)) {
			$mykey = "$mykey/$autoslug";
		}
		
		return "$mykey.html";
	} else {
		return "display_product.php?mykey=$mykey";				
	}
}

function generate_category_url($category) {
	global $SEO_FRIENDLY_URLS;
	global $AUTOSLUG_ENABLED;
		
	if ($category['slug'] != '') {
		$catid = $category['slug'];
	} else {
		$catid = $category['id'];
	}

	if ($SEO_FRIENDLY_URLS) {
		if ($category['cat_type'] == 'auto') {
			if ($AUTOSLUG_ENABLED) {
				return "c/${category['id']}/" . slug($category['name']) . ".html";	
			} else {
				return "show_category.php?category=$url_category" . urlencode($category['name']);
			}			
		} else {
			return "category-$catid.html";			
		}
	}

	return "show_category.php?catid=" . $category['id'];
}


function isRobot() {
  $robots = array('discobot', 'DotBot', 'Nutch', 'Slurp', 'Vagabondo');

	foreach ($robots as $robot) {
		if (strstr($_SERVER['HTTP_USER_AGENT'], $robot)) {
			return 1;
		}
	}
}

function record_hit($db, $clickref, $isFromPPC) {
	global $DEMO_MODE;
	
	if (isRobot()) {
		return;
	}
	
	if ($DEMO_MODE) {
		return;
	}
		
	$statement = $db->prepare('insert into product_hits(mykey, url, site_id, clickref, user_agent, ppc, source_url) values (?,?,?,?,?,?,?)');		
	$statement->bind_param('ssissis', $_GET['id'], $_SERVER['HTTP_REFERER'], $_GET['site_id'], $clickref, $_SERVER['HTTP_USER_AGENT'], $isFromPPC, $_COOKIE['source_url']);
	$result = $statement->execute();	
}

function get_deep_link($db) {
	$id = $db->real_escape_string($_GET['id']);
	$query = "select * from products where mykey = '$id'";
	$result = $db->query($query);
	$product = $result->fetch_object();
	return $product->deep_link;	
}

function add_tracking_to_awin_api_deep_link($deep_link, $clickref) {
	$extra_tracking = str_replace('$kw$', $clickref, '&clickref=$kw$');
	$deep_link .= $extra_tracking;
	return $deep_link;
}

function add_tracking_to_deep_link($deep_link, $clickref) {
	list($feed_id, $product_id) = split('_', $_GET['id']);

	if ($feed_id == 'AWIN') {
		return add_tracking_to_awin_api_deep_link($deep_link, $clickref);
	}
	
	if (! is_numeric($feed_id)) {
		return $deep_link;
	}
	
	$feed = datafeed_find($feed_id);
	$parser = parser_find($feed['parser_id']);

	if ($parser['tracking']) {				
		$extra_tracking = str_replace('$kw$', $clickref, $parser['tracking']);
		$deep_link .= $extra_tracking;
	}	else if ($parser['clickref_function']) {
		$clickref_function = create_function('$deep_link, $clickref', $parser['clickref_function']);
		$deep_link = $clickref_function($deep_link, $clickref);
	}
	
	return $deep_link;
}

function is_valid_id() {
	return (($_GET['id']) && (strpos($_GET['id'], '_')));
}

function redirect($db) {
	if (strstr($_SERVER['REQUEST_URI'], 'url=')) {
		header("Location: " . $_GET['url']);
		exit;		
	}
	
	if (strstr($_SERVER['REQUEST_URI'], 'gclid=')) {
  	setcookie("ppc", "1", time() + 60 * 60 * 24 * 30, "/");
		$isFromPPC = 1;
	}

	if (! $_COOKIE['source_url']) {
		setcookie("source_url", $_SERVER['HTTP_REFERER'], time() + 60 * 60 * 24 * 30, "/");		
	}
	
	$clickref = $_GET['clickref'];
	if (($isFromPPC) || ($_COOKIE['ppc'])) {
		$isFromPPC = 1;
		
		if ($clickref) {
			$clickref = "yes_" . $clickref;
		}
	} else {
		$isFromPPC = 0;
	}
	
	if (is_valid_id())  {
		record_hit($db, $clickref, $isFromPPC);
		$deep_link = get_deep_link($db);

		if ($_GET['clickref']) {
			$deep_link = add_tracking_to_deep_link($deep_link, $clickref);
		}
		
		header("Location: $deep_link");
		exit;
	}
}

function is_boolean_search_mode($query) {
	global $MYSQL_MIN_WORD_LENGTH;
	
	$words = split(' ', $query);
	$boolean_mode = true;
	foreach ($words as $word) {
		if (strlen($word) < $MYSQL_MIN_WORD_LENGTH) {
			$boolean_mode = false;
			break;
		}
	}	
	
	return $boolean_mode;
}

function str_squeeze($query) {
    return trim(ereg_replace(' +', ' ', $query));
}

function get_name_clause($query) {
	$query = str_squeeze($query);
	
	if (is_boolean_search_mode($query)) {
	  $query = '+' . $query;
	  $query = str_replace(' ', ' +', $query);	
		return "match(p.name) against ('$query' in boolean mode)";
	} else {
		return "p.name like '%$query%'";
	}	
}

function str_startswith($source, $prefix) {
   return strncmp($source, $prefix, strlen($prefix)) == 0;
}

function isLegacyQuery($query) {
	return str_startswith($query, 'select * from products where');
}

function updateLegacyQuery($query) {	
	//echo("<br/>Converting legacy query of '$query'<br/> ");
	
	$query = str_replace('mykey', 'p.mykey', $query);	
	$query = str_replace('merchant_name', 'ZZ_MN_ZZ', $query);
	$query = str_replace('name', 'p.name', $query);
	$query = str_replace('ZZ_MN_ZZ', 'merchant_name', $query);
	$query = str_replace('merchant_p.name', 'p.name', $query);
	$query = str_replace('merchant_name', 'p.merchant_name', $query);
	$query = str_replace('display_price', 'p.display_price', $query);
	$query = str_replace('datafeed_id', 'p.datafeed_id', $query);
	$query = str_replace('model_number', 'p.model_number', $query);
	$query = str_replace('description', 'p.description', $query);
	$query = str_replace('select * from products where', 'select p.*, m.*, d.* from datafeeds as d, products as p left outer join meta as m on p.mykey = m.mykey where p.datafeed_id = d.id and', $query);
	
	//echo("<br/>Converted query = '$query'<br/>");		
	
	return $query;
}

function isBooleanQuery($query) {
	return ! str_startswith(strtoupper($query), 'SELECT');
}

function updateQueryWithGeo($query) {
	global $COUNTRY_CODE;

	//echo("<br/>UQWG : Country Code = $COUNTRY_CODE<br/>");
	
  if ((! $COUNTRY_CODE) || (strlen($COUNTRY_CODE) == 2)) {
    return $query;
  }
	//echo("\nO:= $query\n\n");

	if (stristr($query, 'country_code')) {
		return $query;
	}

	if (! strpos($query, 'datafeeds')) {
		return $query;
	}

	$order_by_pos = stripos($query, ' order by');
	if (! $order_by_pos) {
		return $query;
	}
	
	$left_part = substr($query, 0, $order_by_pos);
	$right_part = substr($query, $order_by_pos);
	$new_part = " and d.country_code = '$COUNTRY_CODE'";
	$query = "$left_part$new_part$right_part";
	
	//echo("\nU:= $query\n\n");
	return $query;
	
}

function addNetworkProducts($config, $number, $search_options) {
	global $INSTALL_FOLDER;
	
	#echo("in addNetworkProducts for {$config['classname']} with number of $number<br/>");
	if ((! $number) || (! $search_options)) {
		#echo("NOT searching {$config['classname']}<br/>");
		return;
	}

	$config['max_results'] = $number;
	#echo("Searching {$config['classname']}  for '{$search_options['keywords']}' (MAX:$number)<br/>");

	$network_class_file = $INSTALL_FOLDER . "/admin/models/api/" . $config['filename'] . ".class.php";
	require_once($network_class_file);

	$network_class_name = $config['classname'];
	$network = new $network_class_name($config);
	$products = $network->findProducts($search_options);
	
	$totalAdded = 0;
	foreach ($products as $product) {
		$product_key = $product['network_product_key'];
		
		if (product_key_exists($product_key, $product['merchant_name'])) {
			continue;
		}
		
		$options['product_key'] = stripslashes($product_key);
		$product = $network->getProduct($options);
		$product['autoslug'] = slug($product['name']);
	  product_insert($product);

		#echo "<li>[{$config['classname']}]{$product['name']} from {$product['merchant_name']}</li>";

		$totalAdded++;
		#echo("totalAdded = $totalAdded, max = $number<br/>");
		
		if ($totalAdded >= $number) {
			#echo("Reached max - quitting loop<br/>");
			break;
		}
	}
}

#function get_products($db, $query, $main_url = '', $include_amazon = 0, $amazon_product_name = '', $clickref = '', $site = NULL) {

function get_products($db, $query, $options = array()) {
	
	global $PUBLIC_URL;
	global $AUTO_AMAZON;
	global $INCLUDE_PRODUCT_ID_ON_CLICK_REFERENCES;			
	global $INCLUDE_PRODUCT_NAME_ON_CLICK_REFERENCES;			
	global $DS_GEO_ENABLED;
	global $DS_DEBUG;
 	global $COUNTRY_CODE;
	global $API;
	global $DS_DEFAULT_COUNTRY;
	global $AUTO_PROD_AMAZON;
	global $AUTO_PROD_AWIN;
	global $AUTO_PROD_DAYS;
		
	$default_options = array('main_url' => '',
						'amazon' => '',
						'clickref' => '',
						'site' => NULL,
						'awin' => 0);
	
	$options = array_merge($default_options, $options);
	
	$main_url = $options['main_url'];
	$amazon = $options['amazon'];
	$clickref = $options['clickref'];
	$site = $options['site'];
	$awin = $options['awin'];
	
	$original_query = $query;	
	$search_part = explode('|', $original_query);
	$search_part = $search_part[0];
	
	if (str_startswith($search_part, 'select ')) {
		//echo("starts with select<br/>");
		$match_clause = "(match(p.name) against ('";
		$first_quote = strpos($search_part, $match_clause);
		
		if ($first_quote) {
			$first_quote = $first_quote + strlen($match_clause);
			$second_quote = strpos($search_part, "'", $first_quote + 1);			
		}
		
		//echo("first_quote = $first_quote, second_quote = $second_quote<br/>");
		
		if (($first_quote) && ($second_quote)) {
			$search_part = substr($search_part, $first_quote, ($second_quote - $first_quote));
			//echo("search_part after quotes = $search_part<br/>");
		} else {
			//echo("setting search_part to blank - no quotes<br/>");
			$search_part = '';
		}
	}		

	$words = explode(' ', $search_part);	
	foreach ($words as $word) {
		if ($word[0] != '-') {
			$api_keywords .= ' ' . $word; 
		}
	}
	
	$api_keywords = trim(ereg_replace("[^A-Za-z0-9 ]", "", $api_keywords));
	//echo("api_keywords = $api_keywords<br/>");

	
	if (($api_keywords) && (! api_search_exists($api_keywords, $AUTO_PROD_DAYS))) {
		if ($DS_DEFAULT_COUNTRY == 'GB') {
			$amazon_config = $API['AMA'];
		} elseif ($DS_DEFAULT_COUNTRY == 'US') {
			$amazon_config = $API['AMAUSA'];
		} else {
			$amazon_config = $API["AMACUST"];
		}

		if ($amazon_config) {
			addNetworkProducts($amazon_config, $AUTO_PROD_AMAZON, array('keywords' => $api_keywords, 'category_id' => 'Blended'));			
		}

		addNetworkProducts($API['AWIN'], $AUTO_PROD_AWIN, array('keywords' => $api_keywords));			

		api_search_insert($api_keywords);
		
	}
	
	if ($main_url == '') {
		$main_url = $PUBLIC_URL;
	}
	
	if (isLegacyQuery($query)) {
		$query = updateLegacyQuery($query);
	}

	if (isBooleanQuery($query)) {
		$query = makeSqlQueryFromBoolean($query);
	}
	
	if ($DS_GEO_ENABLED) {
		$query = updateQueryWithGeo($query);
	}

	if (! $db) {
		echo("No DB Object");
		return;
	}

	//echo("QUERY = $query<br/>");

	$result = $db->query($query);
	
	//echo("get_products: '{$result->num_rows}' rows returned from query");
	
	$symbols = array();
	$products = array();

	$numfields = $db->field_count;
	$tfields = Array();
	for ($i = 0; $i < $numfields; $i++) {
	    $field =  $result->fetch_field_direct($i);
	    $tfields[$i] = $field->table. '.' .$field->name;
	}
	
	while ($row = $result->fetch_row()) {
	    $r = Array();
	    for ($i = 0; $i < $numfields; $i++) {
	        $r[$tfields[$i]] = $row[$i];
	    }

	
		if ($DS_DEBUG) {
			echo("<pre>");
			print_r($r);
			echo("</pre>");

			echo("-------------------------------------------<br/><br/>");
		}

		if ($r['m.is_to_be_ignored']) {
			continue;
		}
		
		$mykey = $r['p.mykey'];

		if ($clickref) {
			$prodref = $clickref;
			if ($INCLUDE_PRODUCT_ID_ON_CLICK_REFERENCES) {
				$prodref .= "-$mykey";
			}

			if ($INCLUDE_PRODUCT_NAME_ON_CLICK_REFERENCES) {
				$prodref .= "-" . slug_func($r['p.name']);
			}			
		}
				
		$deep_link = $r['m.deep_link'] ? $r['m.deep_link'] : "index.php?id=$mykey&clickref=$prodref&site_id={$site['id']}";
		
		$in_stock = $r['m.in_stock'] ? $r['m.in_stock'] : $r['p.in_stock'];		
		if ($in_stock == 1) {
			$stock_msg = '<b>In Stock</b>';	
		} elseif ($in_stock == 0) {
			$stock_msg = '<i>Out of Stock</i>';
		} else {
			$stock_msg = "<a href=\"$deep_link\" target=\"_blank\">Check Stock</a>";
		}

		$is_preorder = $r['m.is_preorder'] ? $r['m.is_preorder'] : $r['p.is_preorder'];		
		if ($is_preorder == 1) {
			$preorder_msg = 'Yes';	
		} else {
			$preorder_msg = 'No';
		} 
		$display_product['is_preorder'] = $is_preorder;
		$display_product['preorder_msg'] = $preorder_msg;

		$name = $r['m.name'] ? $r['m.name'] : $r['p.name'];
		$name = strtolower($name);
		$name = ucwords($name);

		$description = $r['m.description'] ? $r['m.description'] : $r['p.description'];		
		$description = str_replace('. ', '.<br/><br/>', $description);
		$description = str_replace(".\r\n", '.<br/><br/>', $description);
		$description = str_replace('\n', '.<br/><br/>', $description);
		
		ob_start();
		$description = eval("?>". stripslashes($description));
		$description = ob_get_clean();
		
				
		$merchant_name = $r['d.merchant_name'];
		$merchant_name = $r['m.merchant_name'] ? $r['m.merchant_name'] : $merchant_name;
		$display_price = ($r['m.display_price'] > 0) ? $r['m.display_price'] : $r['p.display_price'];
		$image_url = $r['m.image_url'] ? $r['m.image_url'] : $r['p.image_url'];
		$delivery_cost = ($r['m.delivery_cost'] > 0) ? $r['m.delivery_cost'] : $r['p.delivery_cost'];
		
		if (($merchant_name == 'Amazon') && ($display_price == 0)) {
			$display_price = "Check!";
			$stock_msg = "<i>N/A</i>";
		}

		if (($r['p.is_custom'] == 1) && ($r['p.deep_link'] == '') && ($r['m.deep_link'] == ''))  {
    	$deep_link = $r['p.image_url'];
		} 

		$autoslug = $r['m.autoslug'] ? $r['m.autoslug'] : $r['p.autoslug'];
		$product_category_id = $r['p.category_id'];

		$display_product['mykey'] = $mykey;
		$display_product['deep_link'] = $deep_link;
		$display_product['merchant_name'] = $merchant_name;
		$display_product['name'] = $name;
		
		if (str_startswith($r['d.merchant_logo'], 'http')) {
			$display_product['merchant_logo'] = $r['d.merchant_logo'];
		} else {
			$display_product['merchant_logo'] = "$PUBLIC_URL/logos/" . $r['d.merchant_logo'];			
		}
		
		$display_product['display_price'] = $display_price;
		$display_product['in_stock'] = $in_stock;
		$display_product['stock_msg'] = $stock_msg;
		$display_product['image_url'] = $image_url;
		$display_product['description'] = $description;
		$display_product['is_custom'] = $r['p.is_custom'];
		$display_product['product_url'] = generate_product_url($mykey, $r['m.slug'], $autoslug);
		$display_product['price_compare_sql'] = $r['m.price_compare_sql'];
		$display_product['original_deep_link'] = $r['p.deep_link'];
		$display_product['delivery_cost'] = $delivery_cost;
		$display_product['product_category_id'] = $product_category_id;
		$display_product['used_price'] = $r['p.used_price'];
		$display_product['userfield1'] = $r['m.userfield1'] ? $r['m.userfield1'] : $r['p.userfield1'];		
		$display_product['userfield2'] = $r['m.userfield2'] ? $r['m.userfield2'] : $r['p.userfield2'];		
		$display_product['userfield3'] = $r['m.userfield3'] ? $r['m.userfield3'] : $r['p.userfield3'];		
		$display_product['userfield4'] = $r['m.userfield4'] ? $r['m.userfield4'] : $r['p.userfield4'];		
		$display_product['merchant_description'] = $r['d.description'];
		$display_product['datafeed_id'] = $r['p.datafeed_id'];
		$display_product['country_code'] = $r['d.country_code'];
		$display_product['rrp_price'] = $r['m.rrp_price'] ? $r['m.rrp_price'] : $r['p.rrp_price'];		
		$display_product['delivery_time'] = $r['m.delivery_time'] ? $r['m.delivery_time'] : $r['p.delivery_time'];		
		$display_product['category'] = $r['p.category'];
		$display_product['subcategory'] = $r['p.subcategory'];
		
		if (! $symbols[$display_product['country_code']]) {
			$symbols[$display_product['country_code']] = currency_code_find_symbol($display_product['country_code']);
		}
		$display_product['currency_prefix'] = $symbols[$display_product['country_code']];			
		
		if ($DS_DEBUG) {
			echo("<pre>");
			//print_r($display_product);
			echo("</pre>");
		}
				
		array_push($products, $display_product);
	}
		
	
	return $products;
}

function display_until($time, $message) {
	
	if (strtotime($time) > time()) {
		echo $message;
	}
}

function get_menu_items($menu_content) {	
	$menu_items = array();

	if (empty($menu_content)) {
		return $menu_items;
	}
	
	$cat_ids = array_keys($menu_content);
	
	
	$result = category_find_with_ids($cat_ids);
	
	for ($i = 0; $i < $result->num_rows; $i++) {
		$category = $result->fetch_array();

		$menu_item = array();
		$menu_item['url'] = generate_category_url($category);
		$menu_item['name'] = $menu_content[$category['id']];

		array_push($menu_items, $menu_item);
	}	
	
	return $menu_items;
}

function get_content_pages($site_id, $order_by) {
	$content_pages = array();	
	$content_result = content_page_find_by_site($site_id, $order_by, CONTENT_PAGE_TYPE, CONTENT_PUBLISHED);

	for ($i = 0; $i < $content_result->num_rows; $i++) {
		array_push($content_pages, $content_result->fetch_array());
	}
	
	return $content_pages;
}

function get_custom_categories($custom_categories_array) {

	$custom_categories = array();
	
	if (empty($custom_categories_array)) {
		return $custom_categories;
	}
	
	foreach ($custom_categories_array as $key) {

		$category = category_find($key);
		
		$custom_category = array();
		$custom_category['url'] = generate_category_url($key, $category['slug']);
		$custom_category['name'] = $category['name'];
		
		array_push($custom_categories, $custom_category);
	}

	return $custom_categories;
}

function get_product_categories($db, $show_all, $cat_array) {
	
	global $AUTOSLUG_ENABLED;

	$categories = array();

	if ($show_all) {
		$result = category_find_all_parents();

	} elseif (count($cat_array) >= 1) {
		$result = category_find_with_ids($cat_array);
	} else {
		return $categories;
	}
		
	for ($i = 0; $i < $result->num_rows; $i++) {
		$category = $result->fetch_array();
		
		if (($category['name'] != '') and ($category['name'])) {
			$category['url'] = generate_category_url($category);			
			array_push($categories, $category);
		}	
	}

	return $categories;
}	

function resultset_to_array($resultset) {
	
	$items = array();
	
	while ($item = $resultset->fetch_array()) {
		array_push($items, $item);
	}
	
	return $items;
}

function get_ds_permalink() {
	$url = $_SERVER['REQUEST_URI'];
	
	if ($url[strlen($url) - 1] != '/') {
		$url .= '/';
	}

	$urlWords = explode('/', $url);		
	$page = $urlWords[sizeof($urlWords) - 2];

	$qmarkpos = strpos($page, '?');
	if ($qmarkpos) {
		$page = substr($page, 0, $qmarkpos);
	} else {
		$page = $page;
	}

	return $page;
}

function get_site_url_from_request() {
	$server = $_SERVER['SERVER_NAME']; 
	$uri = $_SERVER['REQUEST_URI'];
	$uri = substr($uri, 0, strrpos($uri, '/') + 1);
	$url = "http://".$server.$uri;
	
	if (($_GET['autoslug']) && ($_GET['mykey'])) {
		$url = str_replace($_GET['mykey'] . '/', '', $url);
	}
	
	if (($_GET['autoslug']) && ($_GET['catid'])) {
		$url = str_replace("c/{$_GET['catid']}/", '', $url);
	}

	return $url;
}

function get_site_template($permalink) {
	$template = $permalink;
	
	if ($_GET['script']) {
		$template = $_GET['script'];
	} 
	
	if (! strpos($template, '.php')) {
		$template = 'index.php';
	}
	
	$template = str_replace('.php', '', $template);	
	
	return $template;
}

# Quick'n'dirty function for older sites that prefer to use a PHP function call
# to generate product rows inside a HTML table

//($db, $query, $main_url = '', $include_amazon = 0, $amazon_product_name = '', $clickref = '', $site = NULL) {

function display_rows($db, $query, $main_url = '', $mode = 'pcpage', $include_amazon = 0, $amazon_product_name = '', $clickref = '', $site = NULL) {

	global $CURRENCY_PREFIX;
	
	$products = get_products($db, $query, array('main_url' => $main_url, 'clickref' => $clickref, 'site' => $site));
	
	foreach ($products as $product) {
		echo <<<EOROW
<tr>
	<td><a target="_${product['mykey']}" href="${product['deep_link']}"><img 
src="$main_url/${product['merchant_logo']}" height="31" width="88" alt="${product['merchant_name']} Logo" border="0"/></a></td>
	<td><a target="_${product['mykey']}" href="${product['deep_link']}">${product['merchant_name']}</a></td>
	<td><strong>$CURRENCY_PREFIX${product['display_price']}</strong></td>
	<td>${product['name']}</td>
	<td><a target="_${product['mykey']}" href="${product['deep_link']}">Visit Deal!</a></td>
</tr>
EOROW;
	}
	
}

function amazonPresent($products) {
	foreach ($products as $product) {
		if ($product['merchant_name'] == 'Amazon') {
			return true;
		}
	}
	
	return false;
}

function addMatchingAmazonProduct($products, $name, $country_code) {
	global $PUBLIC_URL;
	global $API;
	
	//echo("AMAP: country_code = $country_code");
	if ($country_code == 'GBR') {
		$config = $API['AMA'];
	} elseif ($country_code == 'USA') {
		$config = $API['AMAUSA'];
	} else {
		return $products;
	}
	
	$parsed_xml = ItemSearch($config, 'Blended', $name);

	#echo("<pre>");
	#print_r($parsed_xml);
	#echo("</pre>");
				
	if ($parsed_xml->Items->Item->ASIN) {
		$lookup_xml = ItemLookup($config, $parsed_xml->Items->Item->ASIN);
		$item = $lookup_xml->Items->Item;
		#echo("<pre>");
		#print_r($item);
		#echo("</pre>");
		
	}
			
	if (! $item->ASIN) {
		return $products;
	}
	
	$product = array();
	$product['datafeed_id'] = -1;
	$product['deep_link'] = "index.php?url=" . (string) $item->DetailPageURL;

	$display_price = ($item->OfferSummary->LowestNewPrice->Amount);
	
	if ($display_price == '') {
		$display_price = ($item->ItemAttributes->ListPrice->Amount);		
	}
	
	$display_price = $display_price / 100;		
	$display_price = sprintf("%01.2f", $display_price);

	if ($display_price == 0) {
		$display_price = "Check!";
	}
	
	$product['display_price'] = $display_price;
	$product['used_price'] = ($item->OfferSummary->LowestUsedPrice->Amount) / 100;
	$product['mykey'] = 'AMAZON_AUTO';
	$product['merchant_name'] = 'Amazon';
	$product['merchant_logo'] = "$PUBLIC_URL/logos/amazon.gif";
	$product['name'] = (string) $item->ItemAttributes->Title;
	$product['model_number'] = (string) $item->ItemAttributes->MPN;
	$product['description'] = (string) $item->ItemAttributes->Title;
	$product['thumb_url'] = (string) $item->SmallImage->URL;
	$product['image_url'] = (string) $item->LargeImage->URL;
	$product['delivery_time'] = (string) $item->Offers->Offer->OfferListing->Availability;

	$rrp_price = (string) $item->ItemAttributes->ListPrice->FormattedPrice;
	$rrp_price = substr($rrp_price, 2);
	$product['rrp_price'] = $rrp_price;

	$delivery_time = (string) $item->Offers->Offer->OfferListing->Availability;
	
	if (preg_match("/^Usually/", $delivery_time)) {
		$product['in_stock'] = 1;
	} else {
		$product['in_stock'] = 0;
	}	
	
	$display_product['country_code'] = $country_code;
	$product['currency_prefix'] = $config['currency_symbol'];
	
	if ($product) {
			//echo("<pre>");
			//print_r($product);
			//echo("</pre>");
		array_unshift($products, $product);
	}
	
	return $products;
}	

function convertNameIntoAmazonSearchClause($name) {

	$name = ereg_replace('[^ A-Za-z0-9]', '', html_entity_decode($name)); 
	return str_squeeze($name);
}

function scaleImage($location, $maxw=NULL, $maxh=NULL) {
	$img = @getimagesize($location);

	if (! $img) {
		return array($maxw, $maxh);
	}

	$w = $img[0];
	$h = $img[1];

	$dim = array('w','h');
	foreach($dim AS $val) {
		$max = "max{$val}";
		if (${$val} > ${$max} && ${$max}) {
			$alt = ($val == 'w') ? 'h' : 'w';
		  $ratio = ${$alt} / ${$val};
		  ${$val} = ${$max};
		  ${$alt} = ${$val} * $ratio;
		}
	}

	return array($w, $h);
}

function getSubCategories($parent_id) {
	
	$result = category_find_children($parent_id);
	
	$categories = array();
	
	for ($i = 0; $i < $result->num_rows; $i++) {
		$category = $result->fetch_array();

		if (($category['name'] != '') and ($category['name'])) {
			$category['url'] = generate_category_url($category);			
			array_push($categories, $category);
		}	
	}
	
	return $categories;
}

function download($url, $filename, $user_agent = '') {
	$ch = curl_init($url);
	$fp = fopen($filename, "w");

	curl_setopt($ch, CURLOPT_FILE, $fp);
	curl_setopt($ch, CURLOPT_HEADER, 0);
	
	if ($user_agent != '') {
		curl_setopt($ch, CURLOPT_USERAGENT, $user_agent); 		
	}
	
	curl_exec($ch);
	curl_close($ch);
	fclose($fp);	
}

function build_sql($params) {
	
	global $DS_GEO_ENABLED;
	global $DS_SUPPORTED_COUNTRIES;
	global $DS_DEFAULT_COUNTRY;
  global $COUNTRY_CODE;
	
	//print_r($params);
	
	$params['name'] = stripslashes($params['name']);
	$params['name'] = str_replace("'", '', $params['name']);
	$_POST['name'] = $params['name'];
	$sql = "";	
	
	if (($params['name']) && ($params['namesearchtype'] == 'fuzzy')) {
		$sql .= " and p.name like '%{$params['name']}%'";
	}
	
	if (($params['name']) && ($params['namesearchtype'] == 'boolean')) {
		$query = $params['name'];
		$sql .= " and (match(p.name) against ('$query' in boolean mode))";
	}
		
	if (($params['name']) && ($params['namesearchtype'] == 'exact')) {
		$sql .= " and p.name = '{$params['name']}'";
	}
	
	if ($params['datafeed_id']) {		
		$sql .= " and p.datafeed_id = '{$params['datafeed_id']}'";
	}

	if ($params['category_id']) {		
		$sql .= " and p.category_id = '{$params['category_id']}'";
	}

	if ($params['sub_category_id']) {		
		$sql .= " and p.sub_category_id = '{$params['sub_category_id']}'";
	}

	if ($params['only_merchants']) {
		$sql .= " and p.datafeed_id in ({$params['only_merchants']})";			
	}

	if ($params['exclude_merchants']) {
		$sql .= " and p.datafeed_id not in ({$params['exclude_merchants']})";			
	}

	if ($params['mykey_query']) {
		$sql .= " and p.mykey = '{$params['mykey_query']}'";
	}
	
	if ($params['min_price']) {
		$sql .= " and p.display_price >= '{$params['min_price']}'";
	}
	
	if ($params['max_price']) {
		$sql .= " and p.display_price <= '{$params['max_price']}'";
	}
		
	if ($params['model_number']) {
		$sql .= " and p.model_number like '{$params['model_number']}'";
	}	
	
	if ($params['description']) {
		$sql .= " and p.description like '%{$params['description']}%'";
	}

	for ($i = 1; $i <= 4; $i++) {
		if ($params["userfield$i"]) {
			$sql .= " and p.userfield$i like '%" . $params["userfield$i"] . "%'";
		}
	}

	//echo("in build_sql COUNTRY_CODE = $COUNTRY_CODE<br/>");
	if ($DS_GEO_ENABLED) {
		if ($COUNTRY_CODE) {
			if (strlen($COUNTRY_CODE) == 2) {
				$country_data = find_country_by_code($COUNTRY_CODE);
				$COUNTRY_CODE = $country_data['iso3'];
			}

/*
			if (! in_array($COUNTRY_CODE, $DS_SUPPORTED_COUNTRIES)) {
				$country_data = find_country_by_code($DS_DEFAULT_COUNTRY);
				$COUNTRY_CODE = $country_data['iso3'];
			}
*/
			$sql .= " and d.country_code = '$COUNTRY_CODE'";			
		}		
	}

	if ($params['include_custom'] == "on") {
		$sql .= "";
	} else {
		$sql .= " and p.is_custom = 0";		
	}

	if ($params['in_stock_only'] == "on") {
		$sql .= " and p.in_stock = 1";
	}

	$matches = 0;
	for ($i = 0; $i <= 3; $i++) {		
		
		if (($params['custom_field_name'][$i]) && ($params['custom_field_value'][$i])) {
			$matches++;
			
			if ($matches == 1) {
				$sql .= " and (";
			}
			
			if ($matches > 1) {
				$sql .= ' and ';
			}
			
			$sql .= "(exists (select id from custom_fields cf where p.mykey = cf.mykey and (cf.name = '{$params['custom_field_name'][$i]}' and cf.value = '{$params['custom_field_value'][$i]}' )))";
		}
	}
	
	if ($matches) {
		$distinct = " distinct";
		$custom_field_clause = " left outer join custom_fields as cf on p.mykey = cf.mykey";
		$sql .= ")";
	} else {
		$distinct = "";
		$custom_field_clause = "";
	}
			
	if (! $sql) {
		return '';
	}
	
	$sql = "select$distinct p.*, m.*, d.* from datafeeds as d, products as p left outer join meta as m on p.mykey = m.mykey$custom_field_clause where p.datafeed_id = d.id" . $sql;
	
	if ($params['one_per_merchant'] == "on") {
		$sql .= " group by d.merchant_name";
	}
	
	$sql .= " order by {$params['order_by']}";
	
	if ($params['max_results'] > 0) {
		$sql .= " limit {$params['max_results']}";
	}

	return $sql;
}

function convertQueryStringToParams($query) {

	$query_parts = preg_split("/\|/", $query);	
	$search = $query_parts[0];	
	$option_keyvalues = explode(' ', trim($query_parts[1]));

	$params = array();
	$custom_field_count = 0;
	foreach ($option_keyvalues as $option_keyvalue) {
		list($key, $value) = explode('=', $option_keyvalue);
		$value = str_replace('+', ' ', $value);
		
		if ($key) {
			if ($key == 'cf') {				
				list($custom_key, $custom_value) = explode(':', $value);
				$params['custom_field_name'][$custom_field_count] = $custom_key;
				$params['custom_field_value'][$custom_field_count] = $custom_value;
				$custom_field_count++;
			} else {
				$params[$key] = $value;				
			}
		}
		
	}	
	
	if (! $params['order_field']) {
		$params['order_field'] = "d.merchant_name";
	}
	
	if (! $params['order_direction']) {
		$params['order_direction'] = '';
	}
	
	$params['name'] = trim($search);
	$params['namesearchtype'] = 'boolean';
	$params['order_by'] = trim($params['order_field'] . ' ' . $params['order_direction']);
	
	return $params;
}

function makeSqlQueryFromBoolean($query) {	
	$params = convertQueryStringToParams($query);	
	return build_sql($params);
}

function get_merchant_products($products) {
	$merchants = array();

	if (! $products) {
		return $merchants;
	}
	
	foreach($products as $product) {
		if (array_key_exists($product['merchant_name'], $merchants)) {
			$merchant = $merchants[$product['merchant_name']];
			$merchant['products'][]= $product;
			$merchants[$product['merchant_name']] = $merchant;
		} else {
			$merchant = array();
			$merchant['products'] = array();
			$merchant['products'][]= $product; 
			$merchants[$product['merchant_name']] = $merchant;
		}
	}

	return $merchants;
}

function get_country_code_for_ip() {
	global $DS_GEO_ENABLED;
	global $DS_DEFAULT_COUNTRY;
	global $DS_SUPPORTED_COUNTRIES;

	$code = $DS_DEFAULT_COUNTRY;

	if ($DS_GEO_ENABLED) {

		if ($_SESSION['dstudio_country_code']) {
			//echo("FOUND IN SESSION");
			$code = $_SESSION['dstudio_country_code'];
		} else {
			$ip = $_SERVER['REMOTE_ADDR'];
 		  //echo("IP = $ip<br/>");
			$host = "http://www.geoplugin.net/php.gp?ip=$ip";
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, $host);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			$response = curl_exec($ch);
			curl_close ($ch);

			$data = unserialize($response);
			$code = $data['geoplugin_countryCode'];			
		}
	}

	//$code = 'CA';
	if (! in_array($code, $DS_SUPPORTED_COUNTRIES)) {
		$code = $DS_DEFAULT_COUNTRY;
	}
	
	if (! $code) {
		$code = 'GB';
	}

	//$_SESSION['dstudio_country_code'] = $code;
  //echo("returning code $code<br/>");
	return $code;
}

?>
